@@ -54,6 +54,7 @@ static struct option long_options[] =
 	{"no-commandbar", no_argument, 0, 6},
 	{"no-topbar", no_argument, 0, 7},
 	{"no-cursor", no_argument, 0, 10},
+	{"no-scrollbar", no_argument, 0, 41},
 	{"vertical-cursor", no_argument, 0, 15},
 	{"tabular-cursor", no_argument, 0, 11},
 	{"line-numbers", no_argument, 0, 9},
@@ -67,7 +68,7 @@ static struct option long_options[] =
 	{"double-header", no_argument, 0, 24},
 	{"csv-separator", required_argument, 0, 18},
 	{"border", required_argument, 0, 19},
-	{"no-sigint-exit", no_argument, 0, 21},
+	{"on-sigint-exit", no_argument, 0, 21},
 	{"no-sigint-search-reset", no_argument, 0, 22},
 	{"ni", no_argument, 0, 23},
 	{"log", required_argument, 0, 25},
@@ -99,6 +100,11 @@ static struct option long_options[] =
 	{"blackwhite", no_argument, 0, 'b'},
 	{"freezecols", required_argument, 0, 'c'},
 	{"no-xterm-mouse-mode", no_argument, 0, 40},
+	{"clipboard-app", required_argument, 0, 42},
+	{"no-sleep", no_argument, 0, 43},
+	{"querystream", no_argument, 0, 44},
+	{"menu-always", no_argument, 0, 45},
+	{"no-last-row-search", no_argument, 0, 46},
 	{0, 0, 0, 0}
 };
 
@@ -212,6 +218,59 @@ buildargv(const char *input, int *_argc, char *appname)
 	return argv;
 }
 
+static void
+print_versions(void)
+{
+	fprintf(stdout, "pspg-%s\n", PSPG_VERSION);
+
+#ifdef HAVE_LIBREADLINE
+
+	fprintf(stdout, "with readline (version: 0x%04x)\n", RL_READLINE_VERSION);
+
+#endif
+
+#ifdef COMPILE_MENU
+
+	fprintf(stdout, "with integrated menu\n");
+
+#endif
+
+#ifdef NCURSES_VERSION
+
+	fprintf(stdout, "ncurses version: %s, patch: %ld\n",
+			NCURSES_VERSION,
+			(long) NCURSES_VERSION_PATCH);
+
+#endif
+
+#ifdef HAVE_NCURSESW
+
+	fprintf(stdout, "ncurses with wide char support\n");
+
+#endif
+
+#ifdef NCURSES_WIDECHAR
+
+	fprintf(stdout, "ncurses widechar num: %d\n", NCURSES_WIDECHAR);
+
+#endif
+
+	fprintf(stdout, "wchar_t width: %d, max: %d\n", __SIZEOF_WCHAR_T__, __WCHAR_MAX__);
+
+#ifdef HAVE_POSTGRESQL
+
+	fprintf(stdout, "with postgres client integration\n");
+
+#endif
+
+#ifdef HAVE_INOTIFY
+
+	fprintf(stdout, "with inotify support\n");
+
+#endif
+
+}
+
 bool
 readargs(char **argv,
 		 int argc,
@@ -245,6 +304,7 @@ readargs(char **argv,
 					fprintf(stdout, "  -f, --file=FILE          open file\n");
 					fprintf(stdout, "  -F, --quit-if-one-screen\n");
 					fprintf(stdout, "                           quit if content is one screen\n");
+					fprintf(stdout, "  --clipboard-app=NUM      specify app used by copy to clipboard (1, 2, 3)\n");
 					fprintf(stdout, "  --hold-stream=NUM        can reopen closed FIFO (0, 1, 2)\n");
 					fprintf(stdout, "  --interactive            force interactive mode\n");
 					fprintf(stdout, "  --ignore_file_suffix     don't try to deduce format from file suffix\n");
@@ -253,16 +313,18 @@ readargs(char **argv,
 					fprintf(stdout, "  --no-mouse               don't use own mouse handling\n");
 					fprintf(stdout, "  --no-sigint-search-reset\n");
 					fprintf(stdout, "                           without reset searching on sigint (CTRL C)\n");
-					fprintf(stdout, "  --no_xterm_mouse_mode    don't use optional xterm mouse mode\n");
+					fprintf(stdout, "  --no-sleep               without waits against flickering\n");
+					fprintf(stdout, "  --no-xterm-mouse-mode    don't use optional xterm mouse mode\n");
 					fprintf(stdout, "  --only-for-tables        use std pager when content is not table\n");
-					fprintf(stdout, "  --no-sigint-exit         without exit on sigint(CTRL C or Escape)\n");
+					fprintf(stdout, "  --on-sigint-exit         exit on sigint(CTRL C or Escape)\n");
 					fprintf(stdout, "  --pgcli-fix              try to fix some pgcli related issues\n");
+					fprintf(stdout,  "  --querystream            read queries from stream forever\n");
 					fprintf(stdout, "  --quit-on-f3             exit on F3 like mc viewers\n");
 					fprintf(stdout, "  --rr=ROWNUM              rows reserved for specific purposes\n");
 					fprintf(stdout, "  --stream                 read input forever\n");
 					fprintf(stdout, "  -X, --reprint-on-exit    preserve content after exit\n");
 					fprintf(stdout, "\nOutput format options:\n");
-					fprintf(stdout, "  -a, --ascii              force ascii\n");
+					fprintf(stdout, "  -a, --ascii decor        force ascii\n");
 					fprintf(stdout, "  -b, --blackwhite         black-white style\n");
 					fprintf(stdout, "  -s, --style=N            set color style number (0..%d)\n", MAX_STYLE);
 					fprintf(stdout, "  --bold-labels            row, column labels use bold font\n");
@@ -281,9 +343,12 @@ readargs(char **argv,
 					fprintf(stdout, "  -c, --freezecols=N       freeze N columns (0..9)\n");
 					fprintf(stdout, "  --less-status-bar        status bar like less pager\n");
 					fprintf(stdout, "  --line-numbers           show line number column\n");
+					fprintf(stdout, "  --menu-always            show top bar menu every time\n");
 					fprintf(stdout, "  --no-bars, --no-commandbar, --no-topbar\n");
 					fprintf(stdout, "                           don't show bottom, top bar or both\n");
 					fprintf(stdout, "  --no-cursor              row cursor will be hidden\n");
+					fprintf(stdout, "  --no-last-row-search     don't use the last pattern when starting a new search\n");
+					fprintf(stdout, "  --no-scrollbar           don't show scrollbar\n");
 					fprintf(stdout, "  --no-sound               don't use beep when scroll is not possible\n");
 					fprintf(stdout, "  --tabular-cursor         cursor is visible only when data has table format\n");
 					fprintf(stdout, "  --vertical-cursor        show vertical column cursor\n");
@@ -335,7 +400,7 @@ readargs(char **argv,
 				opts->no_mouse = true;
 				break;
 			case 3:
-				opts->no_sound = true;
+				quiet_mode = true;
 				break;
 			case 4:
 				opts->less_status_bar = true;
@@ -378,7 +443,7 @@ readargs(char **argv,
 				{
 					fprintf(stdout, "The pspg-%s is special pager designed for databases.\n\n", PSPG_VERSION);
 					fprintf(stdout, "Authors:\n");
-					fprintf(stdout, "    2017-2020 Pavel Stehule, Benesov district, Czech Republic\n\n");
+					fprintf(stdout, "    2017-2021 Pavel Stehule, Benesov district, Czech Republic\n\n");
 					fprintf(stdout, "Licence:\n");
 					fprintf(stdout, "    Distributed under BSD licence\n\n");
 
@@ -436,8 +501,9 @@ readargs(char **argv,
 						state->errstr = "csv_header option can be on \"or\" \"off\"";
 						return false;
 					}
+
+					break;
 				}
-				break;
 			case 29:
 				opts->ignore_short_rows = true;
 				break;
@@ -445,8 +511,21 @@ readargs(char **argv,
 				opts->tsv_format = true;
 				break;
 			case 31:
-				opts->nullstr = sstrdup(optarg);
-				break;
+				{
+					char   *nullstr;
+					int		size;
+
+					nullstr = trim_quoted_str(optarg, &size);
+					if (size > 255)
+					{
+						state->errstr = "nullstr is too long (only 255 bytes are allowed)";
+						return false;
+					}
+					else
+						opts->nullstr = sstrndup(nullstr, size);
+
+					break;
+				}
 			case 32:
 				state->ignore_file_suffix = true;
 				break;
@@ -481,60 +560,22 @@ readargs(char **argv,
 			case 39:
 				opts->pgcli_fix = true;
 				break;
-
 			case 40:
 				opts->xterm_mouse_mode =  false;
 				break;
-
+			case 41:
+				opts->show_scrollbar = false;
+				break;
+			case 42:
+				opts->clipboard_app = atoi(optarg);
+				if (opts->clipboard_app < 1 || opts->clipboard_app > 3)
+				{
+					state->errstr = "value of clipboard_app should be 1, 2, or 3";
+					return false;
+				}
+				break;
 			case 'V':
-				fprintf(stdout, "pspg-%s\n", PSPG_VERSION);
-
-#ifdef HAVE_LIBREADLINE
-
-				fprintf(stdout, "with readline (version: 0x%04x)\n", RL_READLINE_VERSION);
-
-#endif
-
-#ifdef COMPILE_MENU
-
-				fprintf(stdout, "with integrated menu\n");
-
-#endif
-
-#ifdef NCURSES_VERSION
-
-				fprintf(stdout, "ncurses version: %s, patch: %ld\n",
-						NCURSES_VERSION,
-						(long) NCURSES_VERSION_PATCH);
-
-#endif
-
-#ifdef HAVE_NCURSESW
-
-				fprintf(stdout, "ncurses with wide char support\n");
-
-#endif
-
-#ifdef NCURSES_WIDECHAR
-
-				fprintf(stdout, "ncurses widechar num: %d\n", NCURSES_WIDECHAR);
-
-#endif
-
-				fprintf(stdout, "wchar_t width: %d, max: %d\n", __SIZEOF_WCHAR_T__, __WCHAR_MAX__);
-
-#ifdef HAVE_POSTGRESQL
-
-				fprintf(stdout, "with postgres client integration\n");
-
-#endif
-
-#ifdef HAVE_INOTIFY
-
-				fprintf(stdout, "with inotify support\n");
-
-#endif
-
+				print_versions();
 				return false;
 			case 'X':
 				state->no_alternate_screen = true;
@@ -604,7 +645,30 @@ readargs(char **argv,
 			case 'd':
 				opts->dbname = sstrdup(optarg);
 				break;
+			case 43:
+				opts->no_sleep = true;
+				break;
+			case 44:
+				opts->querystream = true;
+				state->stream_mode = true;
+				break;
+			case 45:
 
+#ifndef COMPILE_MENU
+
+				state->errstr = "only one file can be browsed";
+				return false;
+
+#else
+
+				opts->menu_always = true;
+				break;
+
+#endif
+
+			case 46:
+				opts->last_row_search = false;
+				break;
 			default:
 				{
 					format_error("Try %s --help\n", argv[0]);
@@ -681,6 +745,12 @@ args_are_consistent(Options *opts, StateData *state)
 		return false;
 	}
 
+	if (opts->query && opts->querystream)
+	{
+		state->errstr = "option --query and --querystream cannot be used together";
+		return false;
+	}
+
 	if (opts->csv_format && opts->tsv_format)
 	{
 		state->errstr = "option --csv and --tsv cannot be used together";
