@@ -16,7 +16,8 @@
 
 int		CTRL_HOME;
 int		CTRL_END;
-
+int		CTRL_SHIFT_HOME;
+int		CTRL_SHIFT_END;
 
 #ifdef NCURSES_EXT_FUNCS
 
@@ -63,6 +64,8 @@ initialize_special_keycodes()
 
 	CTRL_HOME = get_code("kHOM5", 538);
 	CTRL_END = get_code("kEND5", 533);
+	CTRL_SHIFT_HOME = get_code("kHOM6", 537);
+	CTRL_SHIFT_END = get_code("kEND6", 532);
 }
 
 /*
@@ -167,6 +170,10 @@ cmd_string(int cmd)
 			return "SetTheme_Flatwhite";
 		case cmd_SetTheme_RelationalPipes:
 			return "SetThemeRelationalPipes";
+		case cmd_SetTheme_PaperColor:
+			return "SetTheme_PaperColor";
+		case cmd_SetTheme:
+			return "SetTheme";
 
 		case cmd_SaveSetup:
 			return "SaveSetup";
@@ -229,6 +236,10 @@ cmd_string(int cmd)
 			return "ForwardSearch";
 		case cmd_BackwardSearch:
 			return "BackwardSearch";
+		case cmd_ForwardSearchInSelection:
+			return "ForwardSearchInSelection";
+		case cmd_BackwardSearchInSelection:
+			return "BackwardSearchInSelection";
 		case cmd_SearchNext:
 			return "SearchNext";
 		case cmd_SearchPrev:
@@ -243,6 +254,8 @@ cmd_string(int cmd)
 			return "RowNumToggle";
 		case cmd_GotoLine:
 			return "GotoLine";
+		case cmd_GotoLineRel:
+			return "GotoLineRel";
 
 		case cmd_ShowCursor:
 			return "ShowCursor";
@@ -254,6 +267,9 @@ cmd_string(int cmd)
 		case cmd_BoldCursorToggle:
 			return "BoldCursorToggle";
 
+		case cmd_ShowScrollbar:
+			return "ShowScrollBar";
+
 		case cmd_SortAsc:
 			return "SortAsc";
 		case cmd_SortDesc:
@@ -278,12 +294,18 @@ cmd_string(int cmd)
 			return "UseClipboardFormatTSVC";
 		case cmd_UseClipboard_text:
 			return "UseClipboardFormattext";
+		case cmd_UseClipboard_SQL_values:
+			return "UseClipboardFormatSQLValues";
+		case cmd_UseClipboard_pipe_separated:
+			return "UseClipboardFormatPipeSeparated";
 		case cmd_UseClipboard_INSERT:
 			return "UseClipboardFormatINSERT";
 		case cmd_UseClipboard_INSERT_with_comments:
 			return "UseClipboardFormatINSERTwithcomments";
 		case cmd_TogleEmptyStringIsNULL:
 			return "TogleEmptyStringIsNULL";
+		case cmd_SetOwnNULLString:
+			return "SetOwnNULLString";
 		case cmd_Copy:
 			return "Copy";
 		case cmd_CopyAllLines:
@@ -302,15 +324,39 @@ cmd_string(int cmd)
 			return "CopyMarkedLines";
 		case cmd_CopySearchedLines:
 			return "CopySearchedLines";
+		case cmd_CopySelected:
+			return "CopySelected";
+
+		case cmd_Mark:
+			return "Mark";
+		case cmd_MarkColumn:
+			return "MarkColumn";
+		case cmd_MarkAll:
+			return "MarkAll";
+		case cmd_Unmark:
+			return "Unmark";
+		case cmd_Mark_NestedCursorCommand:
+			return "MarkNestedCursorCommand";
+
+		case cmd_BsCommand:
+			return "BsCommand";
 
 		default:
 			return "unknown command";
 	}
 }
 
+bool
+is_cmd_RowNumToggle(int c, bool alt)
+{
+	if (alt && c == 'n')
+		return true;
+
+	return false;
+}
 
 int
-translate_event(int c, bool alt, Options *opts)
+translate_event(int c, bool alt, Options *opts, int *nested_command)
 {
 	if (alt)
 	{
@@ -338,12 +384,26 @@ translate_event(int c, bool alt, Options *opts)
 				return cmd_RawOutputQuit;
 			case 'v':
 				return cmd_ShowVerticalCursor;
+			case '2':
+				return cmd_SaveData;
+			case '3':
+				if (opts->quit_on_f3)
+					return cmd_Quit;
+				else
+					return cmd_Mark;
+				break;
+			case '7':
+				return cmd_ForwardSearch;
 			case '9':
 				return cmd_ShowMenu;
 			case 27:
 				return cmd_Escape;
 			case '0':
 				return cmd_Quit;
+			case '/':
+				return cmd_ForwardSearchInSelection;
+			case '?':
+				return cmd_BackwardSearchInSelection;
 		}
 	}
 	else
@@ -362,7 +422,11 @@ translate_event(int c, bool alt, Options *opts)
 			case KEY_F(3):
 				if (opts->quit_on_f3)
 					return cmd_Quit;
+				else
+					return cmd_Mark;
 				break;
+			case KEY_F(15):
+				return cmd_MarkColumn;
 			case KEY_IC:
 				return cmd_Copy;
 			case KEY_UP:
@@ -450,6 +514,22 @@ translate_event(int c, bool alt, Options *opts)
 			case 'R':
 			case 12:	/* CTRL L */
 				return cmd_Refresh;
+
+			case '\\':
+				return cmd_BsCommand;
+
+			case KEY_SR:
+				*nested_command = cmd_CursorUp;
+				return cmd_Mark_NestedCursorCommand;
+			case KEY_SF:
+				*nested_command = cmd_CursorDown;
+				return cmd_Mark_NestedCursorCommand;
+			case KEY_SNEXT:
+				*nested_command = cmd_PageDown;
+				return cmd_Mark_NestedCursorCommand;
+			case KEY_SPREVIOUS:
+				*nested_command = cmd_PageUp;
+				return cmd_Mark_NestedCursorCommand;
 		}
 	}
 
@@ -457,6 +537,16 @@ translate_event(int c, bool alt, Options *opts)
 		return cmd_CursorFirstRow;
 	else if (c == CTRL_END)
 		return cmd_CursorLastRow;
+	else if (c == CTRL_SHIFT_END)
+	{
+		*nested_command = cmd_CursorLastRow;
+		return cmd_Mark_NestedCursorCommand;
+	}
+	else if (c == CTRL_SHIFT_HOME)
+	{
+		*nested_command = cmd_CursorFirstRow;
+		return cmd_Mark_NestedCursorCommand;
+	}
 
 	return cmd_Invalid;
 }
@@ -516,6 +606,8 @@ theme_get_cmd(int theme)
 			return cmd_SetTheme_Flatwhite;
 		case 22:
 			return cmd_SetTheme_RelationalPipes;
+		case 23:
+			return cmd_SetTheme_PaperColor;
 	};
 
 	return cmd_Invalid;
@@ -576,6 +668,8 @@ cmd_get_theme(int cmd)
 			return 21;
 		case cmd_SetTheme_RelationalPipes:
 			return 22;
+		case cmd_SetTheme_PaperColor:
+			return 23;
 	};
 
 	return 1;
