@@ -69,6 +69,8 @@ typedef enum PspgCommand
 	cmd_SetTheme_TaoLight,
 	cmd_SetTheme_Flatwhite,
 	cmd_SetTheme_RelationalPipes,
+	cmd_SetTheme_PaperColor,
+	cmd_SetTheme,
 	cmd_SaveSetup,
 
 	cmd_Escape,
@@ -100,6 +102,8 @@ typedef enum PspgCommand
 	cmd_SaveAsCSV,
 	cmd_ForwardSearch,
 	cmd_BackwardSearch,
+	cmd_ForwardSearchInSelection,
+	cmd_BackwardSearchInSelection,
 	cmd_SearchNext,
 	cmd_SearchPrev,
 	cmd_SearchColumn,
@@ -107,10 +111,12 @@ typedef enum PspgCommand
 	cmd_ShowBottomBar,
 	cmd_RowNumToggle,
 	cmd_GotoLine,
+	cmd_GotoLineRel,
 	cmd_ShowCursor,
 	cmd_ShowVerticalCursor,
 	cmd_BoldLabelsToggle,
 	cmd_BoldCursorToggle,
+	cmd_ShowScrollbar,
 	cmd_SortAsc,
 	cmd_SortDesc,
 	cmd_OriginalSort,
@@ -120,10 +126,14 @@ typedef enum PspgCommand
 	cmd_SetCopyClipboard,
 	cmd_UseClipboard_CSV,
 	cmd_UseClipboard_TSVC,
+	cmd_UseClipboard_SQL_values,
 	cmd_UseClipboard_text,
+	cmd_UseClipboard_pipe_separated,
 	cmd_UseClipboard_INSERT,
 	cmd_UseClipboard_INSERT_with_comments,
 	cmd_TogleEmptyStringIsNULL,
+	cmd_SetOwnNULLString,
+
 	cmd_Copy,
 	cmd_CopyAllLines,
 	cmd_CopyTopLines,
@@ -132,12 +142,23 @@ typedef enum PspgCommand
 	cmd_CopyLineExtended,
 	cmd_CopyColumn,
 	cmd_CopyMarkedLines,
-	cmd_CopySearchedLines
+	cmd_CopySearchedLines,
+	cmd_CopySelected,
+
+	cmd_Mark,
+	cmd_MarkColumn,
+	cmd_MarkAll,
+	cmd_Unmark,
+	cmd_Mark_NestedCursorCommand,
+
+	cmd_BsCommand
 } PspgCommand;
 
 extern void initialize_special_keycodes();
 extern const char *cmd_string(int cmd);
-extern int translate_event(int c, bool alt, Options *opts);
+extern int translate_event(int c, bool alt, Options *opts, int *nested_command);
+
+extern bool is_cmd_RowNumToggle(int c, bool alt);
 
 extern int cmd_get_theme(int cmd);
 extern int theme_get_cmd(int theme);
