@@ -12,6 +12,7 @@
  */
 #include <ctype.h>
 #include <libgen.h>
+#include <limits.h>
 #include <locale.h>
 #include <stdbool.h>
 #include <stdio.h>
@@ -50,7 +51,6 @@ typedef struct
 	int			size;
 	int			free;
 	LineBuffer *linebuf;
-	bool		force8bit;
 	int			flushed_rows;		/* number of flushed rows */
 	int			maxbytes;
 	bool		printed_headline;
@@ -73,7 +73,7 @@ pb_flush_line(PrintbufType *printbuf)
 {
 	char	   *line;
 
-	if (printbuf->linebuf->nrows == 1000)
+	if (printbuf->linebuf->nrows == LINEBUFFER_LINES)
 	{
 		LineBuffer *nb = smalloc2(sizeof(LineBuffer), "serialize csv output");
 
@@ -229,7 +229,7 @@ pb_print_vertical_header(PrintbufType *printbuf, PrintDataDesc *pdesc, PrintConf
 
 	if (linestyle == 'a')
 	{
-		if (pos == 'm' &&double_header)
+		if (pos == 'm' && double_header)
 		{
 			lhchr = ":";
 			mhchr = ":";
@@ -382,7 +382,7 @@ pb_put_line(char *str, bool multiline, PrintbufType *printbuf)
 				break;
 			}
 
-			chrl = printbuf->force8bit ? 1 : utf8charlen(*ptr);
+			chrl = charlen(ptr);
 			size += chrl;
 			ptr += chrl;
 		}
@@ -496,7 +496,7 @@ pb_print_rowbuckets(PrintbufType *printbuf,
 					{
 						bool	left_align = pdesc->types[j] != 'd';
 
-						if (printbuf->force8bit)
+						if (!use_utf8)
 						{
 							if (multiline)
 							{
@@ -520,11 +520,11 @@ pb_print_rowbuckets(PrintbufType *printbuf,
 						{
 							if (multiline)
 							{
-								width = utf_string_dsplen_multiline(field, SIZE_MAX, &_more_lines, true, NULL, NULL);
+								width = utf_string_dsplen_multiline(field, INT_MAX, &_more_lines, true, NULL, NULL);
 								more_lines |= _more_lines;
 							}
 							else
-								width = utf_string_dsplen(field, SIZE_MAX);
+								width = utf_string_dsplen(field, INT_MAX);
 						}
 
 						spaces = pdesc->widths[j] - width;
@@ -690,7 +690,7 @@ static inline RowBucketType *
 prepare_RowBucket(RowBucketType *rb)
 {
 	/* move row from linebuf to rowbucket */
-	if (rb->nrows >= 1000)
+	if (rb->nrows >= LINEBUFFER_LINES)
 	{
 		RowBucketType *new = smalloc2(sizeof(RowBucketType), "import csv data");
 
@@ -712,7 +712,6 @@ static void
 postprocess_fields(int nfields,
 				   RowType *row,
 				   LinebufType *linebuf,
-				   bool force8bit,
 				   bool ignore_short_rows,
 				   bool reduced_sizes,			/* the doesn't calculate ending zero */
 				   bool *is_multiline_row)
@@ -738,7 +737,7 @@ postprocess_fields(int nfields,
 		if (linebuf->hidden[i])
 			continue;
 
-		if (force8bit)
+		if (!use_utf8)
 		{
 			size_t		cw = 0;
 			char	    *ptr = row->fields[i];
@@ -806,11 +805,10 @@ postprocess_fields(int nfields,
 static void
 postprocess_rows(RowBucketType *rb,
 				 LinebufType *linebuf,
-				 bool force8bit,
 				 char *nullstr)
 {
 	size_t		nullstr_size = strlen(nullstr);
-	size_t		nullstr_width = force8bit ? strlen(nullstr) : (size_t) utf_string_dsplen(nullstr, strlen(nullstr));
+	size_t		nullstr_width = use_utf8 ? (size_t) utf_string_dsplen(nullstr, strlen(nullstr)) : strlen(nullstr);
 
 	while (rb)
 	{
@@ -958,7 +956,6 @@ mark_hidden_columns(LinebufType *linebuf,
 static void
 read_tsv(RowBucketType *rb,
 		 LinebufType *linebuf,
-		 bool force8bit,
 		 FILE *ifile,
 		 bool ignore_short_rows,
 		 Options *opts)
@@ -967,7 +964,8 @@ read_tsv(RowBucketType *rb,
 	int		size = 0;
 	int		nfields = 0;
 	int		c;
-	int		nullstr_size = strlen(opts->nullstr);
+	int		nullstr_size = opts->nullstr ? strlen(opts->nullstr) : 0;
+	char   *nullstr = opts->nullstr ? opts->nullstr : "";
 
 	c = fgetc(ifile);
 	do
@@ -990,7 +988,7 @@ read_tsv(RowBucketType *rb,
 					/* NULL */
 					if (c == 'N')
 					{
-						append_str(linebuf, opts->nullstr);
+						append_str(linebuf, nullstr);
 						size += nullstr_size;
 						goto next_char;
 					}
@@ -1065,7 +1063,6 @@ read_tsv(RowBucketType *rb,
 				postprocess_fields(nfields,
 								   row,
 								   linebuf,
-								   force8bit,
 								   ignore_short_rows,
 								   false,
 								   &multiline);
@@ -1090,14 +1087,13 @@ next_char:
 
 	/* append nullstr to missing columns */
 	if (nullstr_size > 0 && !ignore_short_rows)
-		postprocess_rows(rb, linebuf, force8bit, opts->nullstr);
+		postprocess_rows(rb, linebuf, nullstr);
 }
 
 static void
 read_csv(RowBucketType *rb,
 		 LinebufType *linebuf,
 		 char sep,
-		 bool force8bit,
 		 FILE *ifile,
 		 bool ignore_short_rows,
 		 Options *opts)
@@ -1111,7 +1107,8 @@ read_csv(RowBucketType *rb,
 	int		nfields = 0;
 	int		instr = false;			/* true when csv string is processed */
 	int		c;
-	int		nullstr_size = strlen(opts->nullstr);
+	int		nullstr_size = opts->nullstr ? strlen(opts->nullstr) : 0;
+	char   *nullstr = opts->nullstr ? opts->nullstr : "";
 
 	c = fgetc(ifile);
 
@@ -1209,7 +1206,7 @@ read_csv(RowBucketType *rb,
 					linebuf->sizes[nfields] = nullstr_size;
 					linebuf->starts[nfields++] = pos;
 
-					append_str(linebuf, opts->nullstr);
+					append_str(linebuf, nullstr);
 					pos += nullstr_size;
 				}
 
@@ -1222,7 +1219,7 @@ read_csv(RowBucketType *rb,
 				last_nw = pos;
 			}
 
-			l = force8bit ? 1 : utf8charlen(c);
+			l = use_utf8 ? utf8charlen(c) : 1;
 			if (l > 1)
 			{
 				int		i;
@@ -1272,7 +1269,7 @@ read_csv(RowBucketType *rb,
 				linebuf->sizes[nfields] = nullstr_size;
 				linebuf->starts[nfields++] = pos;
 
-				append_str(linebuf, opts->nullstr);
+				append_str(linebuf, nullstr);
 				pos += nullstr_size;
 			}
 			else
@@ -1321,7 +1318,6 @@ read_csv(RowBucketType *rb,
 			postprocess_fields(nfields,
 							   row,
 							   linebuf,
-							   force8bit,
 							   ignore_short_rows,
 							   true,
 							   &multiline);
@@ -1354,7 +1350,7 @@ next_char:
 
 	/* append nullstr to missing columns */
 	if (nullstr_size > 0 && !ignore_short_rows)
-		postprocess_rows(rb, linebuf, force8bit, opts->nullstr);
+		postprocess_rows(rb, linebuf, nullstr);
 }
 
 /*
@@ -1369,12 +1365,49 @@ read_and_format(Options *opts, DataDesc *desc, StateData *state)
 	PrintConfigType	pconfig;
 	PrintbufType	printbuf;
 	PrintDataDesc	pdesc;
-
-	memset(desc, 0, sizeof(DataDesc));
+	char	   *query = opts->query;
 
 	state->errstr = NULL;
 	state->_errno = 0;
 
+	if (opts->querystream)
+	{
+		SimpleLineBufferIter slbi, *_slbi;
+		char	   *str;
+		ExtStr		estr;
+
+		/* We need to make an query from stored lines */
+		_slbi = init_slbi_ddesc(&slbi, desc);
+		InitExtStr(&estr);
+
+		while (_slbi)
+		{
+			_slbi = slbi_get_line_next(_slbi, &str, NULL);
+			ExtStrAppendNewLine(&estr, str);
+		}
+
+		if (estr.len > 0)
+			query = estr.data;
+		else
+			free(estr.data);
+
+		lb_free(desc);
+
+		if (!query)
+			return false;
+	}
+
+	memset(desc, 0, sizeof(DataDesc));
+
+	if (state->pathname[0])
+	{
+		char	   *name;
+
+		name = basename(state->pathname);
+		strncpy(desc->filename, name, 64);
+		desc->filename[64] = '\0';
+	}
+
 	desc->title[0] = '\0';
 	desc->title_rows = 0;
 	desc->border_top_row = -1;
@@ -1406,25 +1439,36 @@ read_and_format(Options *opts, DataDesc *desc, StateData *state)
 	linebuf.used = 0;
 	linebuf.size = 10 * 1024;
 
-	pconfig.linestyle = (opts->force_ascii_art || opts->force8bit) ? 'a' : 'u';
+	pconfig.linestyle = (opts->force_ascii_art || !use_utf8) ? 'a' : 'u';
 	pconfig.border = opts->border_type;
 	pconfig.double_header = opts->double_header;
 	pconfig.header_mode = opts->csv_header;
 	pconfig.ignore_short_rows = opts->ignore_short_rows;
 
 	rowbuckets.allocated = false;
+	rowbuckets.nrows = 0;
 
-	if (opts->query)
+	if (query)
 	{
-		if (!pg_exec_query(opts, &rowbuckets, &pdesc, &state->errstr))
+		if (!pg_exec_query(opts,
+						   query,
+						   &rowbuckets,
+						   &pdesc,
+						   &state->errstr))
+		{
+			log_row("pgclient error: %s\n", state->errstr);
+
+			if (query != opts->query)
+				free(query);
+
 			return false;
+		}
 	}
 	else if (opts->csv_format)
 	{
 		read_csv(&rowbuckets,
 				 &linebuf,
 				 opts->csv_separator,
-				 opts->force8bit,
 				 state->fp, opts->ignore_short_rows,
 				 opts);
 
@@ -1434,7 +1478,6 @@ read_and_format(Options *opts, DataDesc *desc, StateData *state)
 	{
 		read_tsv(&rowbuckets,
 				 &linebuf,
-				 opts->force8bit,
 				 state->fp,
 				 opts->ignore_short_rows,
 				 opts);
@@ -1448,7 +1491,6 @@ read_and_format(Options *opts, DataDesc *desc, StateData *state)
 	printbuf.free = linebuf.size;
 	printbuf.used = 0;
 	printbuf.linebuf = &desc->rows;
-	printbuf.force8bit = opts->force8bit;
 
 	/* init other printbuf fields */
 	printbuf.printed_headline = false;
@@ -1479,10 +1521,10 @@ read_and_format(Options *opts, DataDesc *desc, StateData *state)
 			desc->headline = desc->rows.rows[headline_rowno];
 			desc->headline_size = strlen(desc->headline);
 
-			if (opts->force8bit)
-				desc->headline_char_size = desc->headline_size;
+			if (use_utf8)
+				desc->headline_char_size = desc->maxx = utf_string_dsplen(desc->headline, INT_MAX);
 			else
-				desc->headline_char_size = desc->maxx = utf_string_dsplen(desc->headline, SIZE_MAX);
+				desc->headline_char_size = desc->headline_size;
 
 			desc->first_data_row = desc->border_head_row + 1;
 
