@@ -111,7 +111,7 @@ flush_bytes(WINDOW *win,
 			}
 			else
 			{
-				int len = opts->force8bit ? 1 : utf8charlen(*rowstr);
+				int len = charlen(rowstr);
 
 				waddnstr(win, rowstr, len);
 				offsetx += utf_dsplen(rowstr);
@@ -181,6 +181,8 @@ print_column_names(WINDOW *win,
 				   int srcx,						/* offset to displayed data */
 				   int vcursor_xmin,				/* xmin in display coordinates */
 				   int vcursor_xmax,				/* xmax in display coordinates */
+				   int selected_xmin,
+				   int selected_xmax,
 				   DataDesc *desc,
 				   Options *opts,
 				   Theme *t)
@@ -194,7 +196,6 @@ print_column_names(WINDOW *win,
 	int		pos = 0;
 	int		bytes;
 	int		chars;
-	bool	force8bit = opts->force8bit;
 
 	attr_t		active_attr = 0;
 	attr_t		new_attr;
@@ -209,16 +210,8 @@ print_column_names(WINDOW *win,
 	/* skip left invisible chars */
 	while (pos < srcx)
 	{
-		if (!force8bit)
-		{
-			bytes = utf8charlen(*ptr);
-			chars = utf_dsplen(ptr);
-		}
-		else
-		{
-			bytes = 1;
-			chars = 1;
-		}
+		bytes = charlen(ptr);
+		chars = dsplen(ptr);
 
 		if (pos + chars > srcx)
 		{
@@ -239,24 +232,31 @@ print_column_names(WINDOW *win,
 	/* position starts from zero again to be comparable with maxx */
 	pos -= srcx;
 
+	if (selected_xmin != -1)
+	{
+		selected_xmin -= srcx;
+		selected_xmax -= srcx;
+
+		if (selected_xmin < 0)
+			selected_xmin = 0;
+	}
+
 	/* for each visible column (position) and defined colum */
 	while (pos < maxx && headline_ptr < headline_end_ptr)
 	{
 		char	column_format = *headline_ptr;
 		bool	is_cursor = vcursor_xmin <= pos && pos <= vcursor_xmax;
+		bool	is_in_range = false;
 
-		if (!force8bit)
-		{
-			bytes = utf8charlen(*ptr);
-			chars = utf_dsplen(ptr);
-		}
-		else
-		{
-			bytes = 1;
-			chars = 1;
-		}
+		is_in_range = selected_xmin != -1 && pos != -1 &&
+						  pos >= selected_xmin && pos <= selected_xmax;
 
-		if (is_cursor )
+		bytes = charlen(ptr);
+		chars = dsplen(ptr);
+
+		if (is_in_range)
+			new_attr = is_cursor ? t->selection_cursor_attr : t->selection_attr;
+		else if (is_cursor)
 			new_attr = column_format == 'd' ? t->cursor_data_attr : t->cursor_line_attr;
 		else
 			new_attr = column_format == 'd' ? t->data_attr : t->line_attr;
@@ -287,6 +287,8 @@ print_column_names(WINDOW *win,
 		pos += chars;
 	}
 
+	wclrtoeol(win);
+
 	wattroff(win, active_attr);
 
 	/* check all column names and print that are visible */
@@ -302,6 +304,9 @@ print_column_names(WINDOW *win,
 		int		act_xmin, act_xmax, act_width;
 		int		overlap_left = 0, overlap_right = 0;
 
+		bool	is_cursor;
+		bool	is_in_range;
+
 		get_column_data_dim(desc, i, &data_x, &data_width);
 
 		if (data_x + data_width < srcx)
@@ -330,7 +335,14 @@ print_column_names(WINDOW *win,
 
 		act_width = act_xmax - act_xmin + 1;
 
-		new_attr = (vcursor_xmin <= act_xmin && act_xmin <= vcursor_xmax) ? t->cursor_data_attr : t->data_attr;
+		is_cursor = vcursor_xmin <= act_xmin && act_xmin <= vcursor_xmax;
+		is_in_range = selected_xmin != -1 &&
+						  act_xmin >= selected_xmin && act_xmin <= selected_xmax;
+
+		if (is_in_range)
+			new_attr = is_cursor ? t->selection_cursor_attr : t->selection_attr;
+		else
+			new_attr = is_cursor ? t->cursor_data_attr : t->data_attr;
 
 		if (colname_width <= act_width)
 		{
@@ -347,19 +359,12 @@ print_column_names(WINDOW *win,
 				/* first n chars */
 				while (*str)
 				{
-					if (!force8bit)
-					{
-						bytes = utf8charlen(*str);
-						chars = utf_dsplen(str);
-					}
-					else
-					{
-						bytes = 1;
-						chars = 1;
-					}
+					bytes = charlen(str);
+					chars = dsplen(str);
 
 					if (width + chars > act_width)
 						break;
+
 					width += chars;
 					str += bytes;
 					colname_size += bytes;
@@ -370,16 +375,8 @@ print_column_names(WINDOW *win,
 				/* last n chars */
 				while (*colname)
 				{
-					if (!force8bit)
-					{
-						bytes = utf8charlen(*colname);
-						chars = utf_dsplen(colname);
-					}
-					else
-					{
-						bytes = 1;
-						chars = 1;
-					}
+					bytes = charlen(colname);
+					chars = dsplen(colname);
 
 					if (colname_width <= act_width)
 						break;
@@ -400,30 +397,27 @@ print_column_names(WINDOW *win,
 LineInfo *
 set_line_info(Options *opts,
 			  ScrDesc *scrdesc,
-			  LineBuffer *lnb,
-			  int lbrn,
+			  DataDesc *desc,
+			  LineBufferMark *lbm,
 			  char *rowstr)
 {
 	LineInfo   *linfo = NULL;
 
-	if (*scrdesc->searchterm == '\0' || !lnb || !rowstr)
+	if (*scrdesc->searchterm == '\0' || !lbm || !rowstr || !lbm->lb)
 		return linfo;
 
-	if (!lnb->lineinfo)
+	if (!lbm->lb->lineinfo)
 	{
+		LineBuffer *lb = lbm->lb;
 		int		i;
 
-		lnb->lineinfo = malloc(1000 * sizeof(LineInfo));
-		if (lnb->lineinfo == NULL)
-			leave("out of memory");
+		lb->lineinfo = smalloc(LINEBUFFER_LINES * sizeof(LineInfo));
 
-		memset(lnb->lineinfo, 0, 1000 * sizeof(LineInfo));
-
-		for (i = 0; i < lnb->nrows; i++)
-			lnb->lineinfo[i].mask = LINEINFO_UNKNOWN;
+		for (i = 0; i < LINEBUFFER_LINES; i++)
+			lb->lineinfo[i].mask = LINEINFO_UNKNOWN;
 	}
 
-	linfo = &lnb->lineinfo[lbrn - 1];
+	linfo = &lbm->lb->lineinfo[lbm->lb_rowno - 1];
 
 	if (linfo->mask & LINEINFO_UNKNOWN)
 	{
@@ -432,6 +426,16 @@ set_line_info(Options *opts,
 		linfo->mask ^= LINEINFO_UNKNOWN;
 		linfo->mask &= ~(LINEINFO_FOUNDSTR | LINEINFO_FOUNDSTR_MULTI);
 
+		/* apply row selection filtr */
+		if (scrdesc->search_rows > 0)
+		{
+			int		rowno = lbm->lineno - desc->first_data_row;
+
+			if (rowno < scrdesc->search_first_row ||
+				rowno > scrdesc->search_first_row + scrdesc->search_rows - 1)
+				return linfo;
+		}
+
 		while (str != NULL)
 		{
 			/*
@@ -441,17 +445,18 @@ set_line_info(Options *opts,
 			 */
 			if (opts->ignore_case || (opts->ignore_lower_case && !scrdesc->has_upperchr))
 			{
-				if (opts->force8bit)
-					str = nstrstr(str, scrdesc->searchterm);
-				else
+
+				if (use_utf8)
 					str = utf8_nstrstr(str, scrdesc->searchterm);
+				else
+					str = nstrstr(str, scrdesc->searchterm);
 			}
 			else if (opts->ignore_lower_case && scrdesc->has_upperchr)
 			{
-				if (opts->force8bit)
-					str = nstrstr_ignore_lower_case(str, scrdesc->searchterm);
-				else
+				if (use_utf8)
 					str = utf8_nstrstr_ignore_lower_case(str, scrdesc->searchterm);
+				else
+					str = nstrstr_ignore_lower_case(str, scrdesc->searchterm);
 			}
 			else
 				/* we can use case sensitive searching (binary comparation) */
@@ -459,6 +464,24 @@ set_line_info(Options *opts,
 
 			if (str != NULL)
 			{
+				/* apply column selection filtr */
+				if (scrdesc->search_columns > 0)
+				{
+					int		bytes = str - rowstr;
+					int		pos;
+
+					pos = use_utf8 ? utf_string_dsplen(rowstr, bytes) : bytes;
+
+					if (pos < scrdesc->search_first_column)
+					{
+						str += charlen(str);
+						continue;
+					}
+
+					if (pos > scrdesc->search_first_column + scrdesc->search_columns - 1)
+						return linfo;
+				}
+
 				if (linfo->mask & LINEINFO_FOUNDSTR)
 				{
 					/* When we detect multi occurrence, then stop searching */
@@ -468,10 +491,11 @@ set_line_info(Options *opts,
 				else
 				{
 					linfo->mask |= LINEINFO_FOUNDSTR;
-					if (opts->force8bit)
-						linfo->start_char = str - rowstr;
+
+					if (use_utf8)
+						linfo->start_char = utf_string_dsplen(rowstr, str - rowstr);
 					else
-						linfo->start_char = utf8len_start_stop(rowstr, str);
+						linfo->start_char = str - rowstr;
 				}
 
 				str += scrdesc->searchterm_size;
@@ -482,6 +506,95 @@ set_line_info(Options *opts,
 	return linfo;
 }
 
+/*
+ * Draw scrollbar to related window.
+ */
+static void
+draw_scrollbar_win(WINDOW *win,
+				   Theme *t,
+				   ScrDesc *scrdesc,
+				   Options *opts)
+{
+	int		i;
+
+	werase(win);
+
+	wattron(win, t->scrollbar_attr);
+
+	for (i = 0; i < scrdesc->scrollbar_maxy; i++)
+		waddch(win, ACS_CKBOARD);
+
+	wattroff(win, t->scrollbar_attr);
+	wattron(win, t->scrollbar_arrow_attr);
+
+	if (t->scrollbar_use_arrows)
+	{
+		if (!use_utf8 || opts->force_ascii_art)
+		{
+			mvwaddch(win, 0, 0, ACS_UARROW);
+			mvwaddch(win, scrdesc->scrollbar_maxy - 1, 0, ACS_DARROW);
+		}
+		else
+		{
+			/* 🠕 🠗 */
+			mvwprintw(win, 0, 0, "%lc", L'\x1F815');
+			mvwprintw(win, scrdesc->scrollbar_maxy - 1, 0, "%lc", L'\x1F817');
+		}
+	}
+	else
+	{
+		/* ▲ ▼ */
+		mvwprintw(win, 0, 0, "%lc", L'\x25b2');
+		mvwprintw(win, scrdesc->scrollbar_maxy - 1, 0, "%lc", L'\x25bc');
+	}
+
+	wattroff(win, t->scrollbar_arrow_attr);
+	wattron(win, scrdesc->scrollbar_mode ? t->scrollbar_active_slider_attr : t->scrollbar_slider_attr);
+
+	if (!t->scrollbar_slider_symbol)
+	{
+		/* draw slider */
+		for (i = 0; i < scrdesc->slider_size; i++)
+			mvwprintw(win, scrdesc->slider_min_y + i, 0, " ");
+	}
+	else
+		mvwaddch(win, scrdesc->slider_min_y, 0, t->scrollbar_slider_symbol);
+
+	wattroff(win, scrdesc->scrollbar_mode ? t->scrollbar_active_slider_attr : t->scrollbar_slider_attr);
+}
+
+/*
+ * Return true when pos is over some searched patterns specified by
+ * positions cache or lineinfo position. This function can be called
+ * only when the row is pattern row (linfo is valid).
+ */
+static bool
+is_in_searched_pattern(int pos,
+					   ScrDesc *scrdesc,
+					   LineInfo *linfo,
+					   int positions[100][2],
+					   int npositions)
+{
+	if (linfo && linfo->mask & LINEINFO_FOUNDSTR_MULTI)
+	{
+		int		i;
+
+		for (i = 0; i < npositions; i++)
+		{
+			if (pos >= positions[i][0] && pos < positions[i][1])
+				return true;
+		}
+	}
+	else
+	{
+		if (pos >= linfo->start_char &&
+			 pos < linfo->start_char + scrdesc->searchterm_char_size)
+			return true;
+	}
+
+	return false;
+}
+
 void
 window_fill(int window_identifier,
 			int srcy,
@@ -489,25 +602,34 @@ window_fill(int window_identifier,
 			int cursor_row,					/* row of row cursor */
 			int vcursor_xmin,				/* xmin in display coordinates */
 			int vcursor_xmax,				/* xmax in display coordinates */
+			int selected_xmin,
+			int selected_xmax,
 			DataDesc *desc,
 			ScrDesc *scrdesc,
 			Options *opts)
 {
 	int			maxy, maxx;
 	int			row;
-	LineBuffer *lnb = &desc->rows;
-	int			lnb_row;
+	LineBufferIter lbi;
+	LineBufferMark lbm;
 	attr_t		active_attr;
 	attr_t		pattern_fix;
 	int			srcy_bak = srcy;
 	char		*free_row;
 	WINDOW		*win;
 	Theme		*t;
+
 	bool		is_footer = window_identifier == WINDOW_FOOTER;
 	bool		is_fix_rows = window_identifier == WINDOW_LUC || window_identifier == WINDOW_FIX_ROWS;
 	bool		is_rownum = window_identifier == WINDOW_ROWNUM;
 	bool		is_rownum_luc = window_identifier == WINDOW_ROWNUM_LUC;
 	bool		is_fix_rows_only = window_identifier == WINDOW_FIX_ROWS;
+	bool		is_scrollbar = window_identifier == WINDOW_VSCROLLBAR;
+	bool		is_selectable = window_identifier == WINDOW_ROWS ||
+								window_identifier == WINDOW_LUC ||
+								window_identifier == WINDOW_FIX_COLS ||
+								window_identifier == WINDOW_FIX_ROWS ||
+								window_identifier == WINDOW_FOOTER;
 
 	win = scrdesc->wins[window_identifier];
 	t = &scrdesc->themes[window_identifier];
@@ -532,14 +654,14 @@ window_fill(int window_identifier,
 		return;
 	}
 
-	/* skip first x LineBuffers */
-	while (srcy > 1000)
+	if (is_scrollbar)
 	{
-		lnb = lnb->next;
-		srcy -= 1000;
+		draw_scrollbar_win(win, t, scrdesc, opts);
+		return;
 	}
 
-	lnb_row = srcy;
+	init_lbi_ddesc(&lbi, desc, srcy);
+
 	row = 0;
 
 	getmaxyx(win, maxy, maxx);
@@ -558,51 +680,18 @@ window_fill(int window_identifier,
 		char		buffer[10];
 		int			positions[100][2];
 		int			npositions = 0;
+		int			is_in_range = false;
+		int			rowno = row + srcy_bak + 1 - desc->first_data_row;
 
 		is_cursor_row = (!opts->no_cursor && row == cursor_row);
 
-		if (desc->order_map)
-		{
-			int		rowno = row + srcy_bak;
+		(void) lbi_set_mark_next(&lbi, &lbm);
 
-			if (rowno <= desc->last_row)
-			{
-				MappedLine *mp = &desc->order_map[rowno];
-
-				lnb = mp->lnb;
-				lnb_row = mp->lnb_row;
-				rowstr = lnb->rows[lnb_row];
-				lineinfo = lnb->lineinfo ? &lnb->lineinfo[lnb_row] : NULL;
-
-				line_is_valid = true;
-			}
-		}
-		else
-		{
-			if (lnb_row == 1000)
-			{
-				lnb = lnb ? lnb->next : NULL;
-				lnb_row = 0;
-			}
-
-			if (lnb != NULL && lnb_row < lnb->nrows)
-			{
-				rowstr = lnb->rows[lnb_row];
-				if (lnb->lineinfo != NULL)
-					lineinfo = &lnb->lineinfo[lnb_row];
-				else
-					lineinfo = NULL;
-				lnb_row += 1;
-
-				line_is_valid = true;
-			}
-		}
+		line_is_valid = lbm_get_line(&lbm, &rowstr, &lineinfo, NULL);
 
 		/* when rownum is printed, don't process original text */
 		if (is_rownum && line_is_valid)
 		{
-			int rowno = row + srcy_bak + 1 - desc->first_data_row;
-
 			snprintf(buffer, sizeof(buffer), "%*d ", maxx - 1, rowno);
 			rowstr = buffer;
 		}
@@ -610,7 +699,7 @@ window_fill(int window_identifier,
 		is_bookmark_row = (lineinfo != NULL && (lineinfo->mask & LINEINFO_BOOKMARK) != 0) ? true : false;
 
 		if (!is_fix_rows && *scrdesc->searchterm != '\0' && !opts->no_highlight_search)
-			lineinfo = set_line_info(opts, scrdesc, lnb, lnb_row, rowstr);
+			lineinfo = set_line_info(opts, scrdesc, desc, &lbm, rowstr);
 
 		is_pattern_row = (lineinfo != NULL && (lineinfo->mask & LINEINFO_FOUNDSTR) != 0) ? true : false;
 
@@ -627,7 +716,22 @@ window_fill(int window_identifier,
 
 				if (str != NULL)
 				{
-					positions[npositions][0] = opts->force8bit ? (size_t) (str - rowstr) : utf8len_start_stop(rowstr, str);
+					int		position = use_utf8 ? utf_string_dsplen(rowstr, str - rowstr) : (int) (str - rowstr);
+
+					/* apply column selection filtr */
+					if (scrdesc->search_columns > 0)
+					{
+						if (position < scrdesc->search_first_column)
+						{
+							str += charlen(str);
+							continue;
+						}
+
+						if (position > scrdesc->search_first_column + scrdesc->search_columns - 1)
+							break;
+					}
+
+					positions[npositions][0] = position;
 					positions[npositions][1] = positions[npositions][0] + scrdesc->searchterm_char_size;
 
 					/* don't search more if we are over visible part */
@@ -718,7 +822,7 @@ window_fill(int window_identifier,
 			if (desc->is_expanded_mode)
 			{
 				fix_line_attr_style = effective_row >= desc->border_bottom_row;
-				is_expand_head = is_expanded_header(opts, rowstr, &ei_min, &ei_max);
+				is_expand_head = is_expanded_header(rowstr, &ei_min, &ei_max);
 				if (is_expand_head)
 				{
 					if (scrdesc->first_rec_title_y == -1)
@@ -749,42 +853,38 @@ window_fill(int window_identifier,
 			 */
 			if (is_fix_rows_only && rowstr == desc->namesline )
 			{
-				print_column_names(win, srcx, vcursor_xmin, vcursor_xmax, desc, opts, t);
+				int		loc_selected_xmin = -1;
+				int		loc_selected_xmax = -1;
+
+				/* mark columns names only when columns are selected */
+				if (selected_xmin != -1 && scrdesc->selected_first_row == -1)
+				{
+					loc_selected_xmin = selected_xmin;
+					loc_selected_xmax = selected_xmax;
+				}
+
+				print_column_names(win, srcx,
+								   vcursor_xmin, vcursor_xmax,
+								   loc_selected_xmin, loc_selected_xmax,
+								   desc, opts, t);
 				continue;
 			}
 
 			/* skip first srcx chars */
 			i = srcx;
 			left_spaces = 0;
-			if (opts->force8bit)
+
+			while(i > 0)
 			{
-				while(i > 0)
+				if (*rowstr != '\0' && *rowstr != '\n')
 				{
-					if (*rowstr != '\0' && *rowstr != '\n')
-					{
-						i -= 1;
-						rowstr += 1;
-						if (i < 0)
-							left_spaces = -i;
-					}
-					else
-						break;
-				}
-			}
-			else
-			{
-				while(i > 0)
-				{
-					if (*rowstr != '\0' && *rowstr != '\n')
-					{
-						i -= utf_dsplen(rowstr);
-						rowstr += utf8charlen(*rowstr);
-						if (i < 0)
-							left_spaces = -i;
-					}
-					else
-						break;
+					i -= dsplen(rowstr);
+					rowstr += charlen(rowstr);
+					if (i < 0)
+						left_spaces = -i;
 				}
+				else
+					break;
 			}
 
 			/* Fix too hungry cutting when some multichar char is removed */
@@ -822,6 +922,28 @@ window_fill(int window_identifier,
 					int		pos = (i != -1) ? srcx + i : -1;
 					bool	skip_char = false;
 
+					is_in_range = false;
+
+					if (is_selectable && scrdesc->selected_first_row != -1)
+					{
+						if (rowno >= scrdesc->selected_first_row + 1 &&
+							rowno < scrdesc->selected_first_row + 1 + scrdesc->selected_rows)
+						{
+							if (selected_xmin != -1 && pos != -1)
+							{
+								if (pos >= selected_xmin && pos <= selected_xmax)
+									is_in_range = true;
+							}
+							else
+								is_in_range = true;
+						}
+					}
+					else if (is_selectable && selected_xmin != -1 && pos != -1)
+					{
+						if (pos >= selected_xmin && pos <= selected_xmax)
+							is_in_range = true;
+					}
+
 					if (i != -1 && vcursor_xmin <= i && i <= vcursor_xmax)
 					{
 						is_cross_cursor = is_cursor_row;
@@ -879,7 +1001,14 @@ window_fill(int window_identifier,
 							}
 						}
 
-						if (is_cross_cursor)
+						if (is_in_range)
+						{
+							new_attr = is_cursor ? t->selection_cursor_attr : t->selection_attr;
+							if (is_pattern_row && !is_cursor &&
+									is_in_searched_pattern(pos, scrdesc, lineinfo, positions, npositions))
+								new_attr = new_attr ^ A_REVERSE;
+						}
+						else if (is_cross_cursor)
 						{
 							new_attr = column_format == 'd' ? t->cross_cursor_attr : t->cross_cursor_line_attr;
 						}
@@ -899,26 +1028,10 @@ window_fill(int window_identifier,
 							else if (pos < desc->headline_char_size)
 								new_attr = column_format == 'd' ? t->pattern_data_attr : t->pattern_line_attr;
 
-							if ((new_attr == t->pattern_data_attr || new_attr == t->pattern_vertical_cursor_attr) && pos >= lineinfo->start_char)
+							if (new_attr == t->pattern_data_attr || new_attr == t->pattern_vertical_cursor_attr)
 							{
-								if ((lineinfo->mask & LINEINFO_FOUNDSTR_MULTI) != 0)
-								{
-									int		j;
-
-									for (j = 0; j < npositions; j++)
-									{
-										if (pos >= positions[j][0] && pos < positions[j][1])
-										{
-											new_attr = t->found_str_attr;
-											break;
-										}
-									}
-								}
-								else
-								{
-									if (pos < lineinfo->start_char + scrdesc->searchterm_char_size)
-										new_attr = t->found_str_attr;
-								}
+								if (is_in_searched_pattern(pos, scrdesc, lineinfo, positions, npositions))
+									new_attr = t->found_str_attr;
 							}
 						}
 						else if (is_footer)
@@ -936,26 +1049,10 @@ window_fill(int window_identifier,
 							if (is_found_row && pos >= scrdesc->found_start_x &&
 									pos < scrdesc->found_start_x + scrdesc->searchterm_char_size)
 								new_attr = new_attr ^ ( A_REVERSE | pattern_fix );
-							else if (is_pattern_row && pos >= lineinfo->start_char)
+							else if (is_pattern_row)
 							{
-								if ((lineinfo->mask & LINEINFO_FOUNDSTR_MULTI) != 0)
-								{
-									int		j;
-
-									for (j = 0; j < npositions; j++)
-									{
-										if (pos >= positions[j][0] && pos < positions[j][1])
-										{
-											new_attr = t->cursor_pattern_attr;
-											break;
-										}
-									}
-								}
-								else
-								{
-									if (pos < lineinfo->start_char + scrdesc->searchterm_char_size)
-										new_attr = t->cursor_pattern_attr;
-								}
+								if (is_in_searched_pattern(pos, scrdesc, lineinfo, positions, npositions))
+									new_attr = t->cursor_pattern_attr;
 							}
 						}
 
@@ -1004,18 +1101,17 @@ window_fill(int window_identifier,
 						{
 							attr_t		new_attr;
 
-							if (is_cross_cursor)
-							{
+							if (is_in_range)
+								new_attr = is_cursor ? t->selection_cursor_attr : t->selection_attr;
+
+							else if (is_cross_cursor)
 								new_attr = t->cross_cursor_line_attr;
-							}
+
 							else if (is_cursor)
-							{
 								new_attr = t->cursor_line_attr;
-							}
+
 							else
-							{
 								new_attr = t->line_attr;
-							}
 
 							if (new_attr != active_attr)
 							{
@@ -1049,25 +1145,14 @@ window_fill(int window_identifier,
 
 					if (*ptr != '\0')
 					{
-						if (opts->force8bit)
-						{
-							i += 1;
-							ptr += 1;
+						int dlen = dsplen(ptr);
+						int len  = charlen(ptr);
 
-							if (!skip_char)
-								bytes += 1;
-						}
-						else
-						{
-							int dsplen = utf_dsplen(ptr);
-							int len  = utf8charlen(*ptr);
+						i = (dlen != -1 && i != -1) ? i + dlen : -1;
+						ptr += len;
 
-							i = dsplen != -1 && i != -1 ? i + dsplen : -1;
-							ptr += len;
-
-							if (!skip_char)
-								bytes += len;
-						}
+						if (!skip_char)
+							bytes += len;
 					}
 					else
 					{
@@ -1117,14 +1202,23 @@ window_fill(int window_identifier,
 			/* draw cursor or bookmark line to screen end of line */
 			if (i < maxx)
 			{
-				if (is_cursor_row && !is_bookmark_row)
-					mvwchgat(win, row - 1, i, -1, t->cursor_data_attr, PAIR_NUMBER(t->cursor_data_attr), 0);
+				attr_t		attr = 0;
+
+				if (is_in_range && is_cursor_row)
+					attr = t->selection_cursor_attr;
+				else if (is_in_range && !is_cursor_row)
+					attr = t->selection_attr;
+				else if (is_cursor_row && !is_bookmark_row)
+					attr = t->cursor_data_attr;
 				else if (!is_cursor_row && is_bookmark_row)
-					mvwchgat(win, row - 1, i, -1, t->bookmark_data_attr, PAIR_NUMBER(t->bookmark_data_attr), 0);
+					attr = t->bookmark_data_attr;
 				else if (is_cursor_row && is_bookmark_row)
-					mvwchgat(win, row - 1, i, -1, t->cursor_bookmark_attr, PAIR_NUMBER(t->cursor_bookmark_attr), 0);
+					attr = t->cursor_bookmark_attr;
 				else if (!is_cursor_row && is_pattern_row)
-					mvwchgat(win, row - 1, i, -1, t->pattern_data_attr, PAIR_NUMBER(t->pattern_data_attr), 0);
+					attr = t->pattern_data_attr;
+
+				if (attr)
+					mvwchgat(win, row - 1, i, -1, attr, PAIR_NUMBER(attr), 0);
 			}
 
 			if (free_row != NULL)
@@ -1194,7 +1288,6 @@ static void
 draw_rectange(int offsety, int offsetx,			/* y, x offset on screen */
 			int maxy, int maxx,				/* size of visible rectangle */
 			int srcy, int srcx,				/* offset to displayed data */
-			Options *opts,
 			DataDesc *desc,
 			attr_t data_attr,				/* colors for data (alphanums) */
 			attr_t line_attr,				/* colors for borders */
@@ -1202,19 +1295,12 @@ draw_rectange(int offsety, int offsetx,			/* y, x offset on screen */
 			bool clreoln)					/* force clear to eoln */
 {
 	int			row;
-	LineBuffer *lnb = &desc->rows;
-	int			lnb_row;
+	LineBufferIter lbi;
 	attr_t		active_attr;
 	int			srcy_bak = srcy;
 
-	/* skip first x LineBuffers */
-	while (srcy > 1000)
-	{
-		lnb = lnb->next;
-		srcy -= 1000;
-	}
+	init_lbi_ddesc(&lbi, desc, srcy);
 
-	lnb_row = srcy;
 	row = 0;
 
 	if (offsety)
@@ -1227,33 +1313,7 @@ draw_rectange(int offsety, int offsetx,			/* y, x offset on screen */
 		char	   *ptr;
 		char	   *rowstr = NULL;
 
-		if (desc->order_map)
-		{
-			int		rowno = row + srcy_bak;
-
-			if (rowno <= desc->last_row)
-			{
-				MappedLine *mp = &desc->order_map[rowno];
-
-				lnb = mp->lnb;
-				lnb_row = mp->lnb_row;
-				rowstr = lnb->rows[lnb_row];
-			}
-		}
-		else
-		{
-			if (lnb_row == 1000)
-			{
-				lnb = lnb ? lnb->next : NULL;
-				lnb_row = 0;
-			}
-
-			if (lnb != NULL && lnb_row < lnb->nrows)
-			{
-				rowstr = lnb->rows[lnb_row];
-				lnb_row += 1;
-			}
-		}
+		(void) lbi_get_line_next(&lbi, &rowstr, NULL, NULL);
 
 		active_attr = line_attr;
 		printf("%s", ansi_attr(active_attr));
@@ -1273,7 +1333,7 @@ draw_rectange(int offsety, int offsetx,			/* y, x offset on screen */
 			if (desc->is_expanded_mode)
 			{
 				fix_line_attr_style = effective_row >= desc->border_bottom_row;
-				is_expand_head = is_expanded_header(opts, rowstr, &ei_min, &ei_max);
+				is_expand_head = is_expanded_header(rowstr, &ei_min, &ei_max);
 			}
 			else
 			{
@@ -1289,35 +1349,18 @@ draw_rectange(int offsety, int offsetx,			/* y, x offset on screen */
 			/* skip first srcx chars */
 			i = srcx;
 			left_spaces = 0;
-			if (opts->force8bit)
+
+			while(i > 0)
 			{
-				while(i > 0)
+				if (*rowstr != '\0' && *rowstr != '\n')
 				{
-					if (*rowstr != '\0' && *rowstr != '\n')
-					{
-						i -= 1;
-						rowstr += 1;
-						if (i < 0)
-							left_spaces = -i;
-					}
-					else
-						break;
-				}
-			}
-			else
-			{
-				while(i > 0)
-				{
-					if (*rowstr != '\0' && *rowstr != '\n')
-					{
-						i -= utf_dsplen(rowstr);
-						rowstr += utf8charlen(*rowstr);
-						if (i < 0)
-							left_spaces = -i;
-					}
-					else
-						break;
+					i -= dsplen(rowstr);
+					rowstr += charlen(rowstr);
+					if (i < 0)
+						left_spaces = -i;
 				}
+				else
+					break;
 			}
 
 			/* Fix too hungry cutting when some multichar char is removed */
@@ -1386,7 +1429,6 @@ draw_rectange(int offsety, int offsetx,			/* y, x offset on screen */
 							{
 								if (bytes > 0)
 								{
-									//waddnstr(win, rowstr, bytes);
 									printf("%.*s", bytes, rowstr);
 									rowstr += bytes;
 									bytes = 0;
@@ -1401,8 +1443,9 @@ draw_rectange(int offsety, int offsetx,			/* y, x offset on screen */
 
 					if (*ptr != '\0' && *ptr != '\n')
 					{
-						int len  = opts->force8bit ? 1 : utf8charlen(*ptr);
-						i += opts->force8bit ? 1 : utf_dsplen(ptr);
+						int len  = charlen(ptr);
+
+						i += dsplen(ptr);
 						ptr += len;
 						bytes += len;
 					}
@@ -1463,7 +1506,7 @@ draw_data(Options *opts, ScrDesc *scrdesc, DataDesc *desc,
 			draw_rectange(scrdesc->fix_rows_rows, 0,
 						  scrdesc->rows_rows, scrdesc->fix_cols_cols,
 						  first_data_row + first_row - fix_rows_offset, 0,
-						  opts, desc,
+						  desc,
 						  COLOR_PAIR(4) | A_BOLD, 0, COLOR_PAIR(8) | A_BOLD,
 						  false);
 		}
@@ -1476,7 +1519,7 @@ draw_data(Options *opts, ScrDesc *scrdesc, DataDesc *desc,
 			draw_rectange(0, scrdesc->fix_cols_cols,
 						  scrdesc->fix_rows_rows, size.ws_col - scrdesc->fix_cols_cols,
 						  desc->title_rows + fix_rows_offset, scrdesc->fix_cols_cols + cursor_col,
-						  opts, desc,
+						  desc,
 						  COLOR_PAIR(4) | A_BOLD, 0, COLOR_PAIR(8) | A_BOLD,
 						  true);
 		}
@@ -1489,7 +1532,7 @@ draw_data(Options *opts, ScrDesc *scrdesc, DataDesc *desc,
 			draw_rectange(0, 0,
 						  scrdesc->fix_rows_rows, scrdesc->fix_cols_cols,
 						  desc->title_rows + fix_rows_offset, 0,
-						  opts, desc,
+						  desc,
 						  COLOR_PAIR(4) | A_BOLD, 0, COLOR_PAIR(8) | A_BOLD,
 						  false);
 		}
@@ -1502,7 +1545,7 @@ draw_data(Options *opts, ScrDesc *scrdesc, DataDesc *desc,
 			draw_rectange(scrdesc->fix_rows_rows, scrdesc->fix_cols_cols,
 						  scrdesc->rows_rows, size.ws_col - scrdesc->fix_cols_cols,
 						  first_data_row + first_row - fix_rows_offset, scrdesc->fix_cols_cols + cursor_col,
-						  opts, desc,
+						  desc,
 						  opts->theme == 2 ? 0 | A_BOLD : 0,
 						  opts->theme == 2 && (desc->headline_transl == NULL) ? A_BOLD : 0,
 						  COLOR_PAIR(8) | A_BOLD,
@@ -1517,7 +1560,7 @@ draw_data(Options *opts, ScrDesc *scrdesc, DataDesc *desc,
 			draw_rectange(scrdesc->fix_rows_rows + scrdesc->rows_rows, 0,
 						  scrdesc->footer_rows, scrdesc->maxx,
 						  first_data_row + first_row + scrdesc->rows_rows - fix_rows_offset, footer_cursor_col,
-						  opts, desc,
+						  desc,
 						  COLOR_PAIR(9), 0, 0, true);
 		}
 
