@@ -296,7 +296,7 @@ chr_casexfrm(ST_MENU_CONFIG *config, char *str)
 
 		fold  = utf8_tofold((const char *) buffer2);
 
-		*((int *) buffer) = fold;
+		memcpy(buffer, &fold, sizeof(int));
 		buffer[sizeof(int)] = '\0';
 
 		result = strdup(buffer);
@@ -1563,6 +1563,16 @@ _st_menu_driver(struct ST_MENU *menu, int c, bool alt, MEVENT *mevent,
 		(menu->focus == ST_MENU_FOCUS_NONE))
 		goto post_process;
 
+#if NCURSES_MOUSE_VERSION > 1
+
+	/* use mouse wheel only with FULL FOCUS */
+	if (c == KEY_MOUSE &&
+		(mevent->bstate & BUTTON4_PRESSED || mevent->bstate & BUTTON5_PRESSED) &&
+		menu->focus != ST_MENU_FOCUS_FULL)
+		goto post_process;
+
+#endif
+
 	/*
 	 * Propagate event to nested active object first. When nested object would be
 	 * closed, close it. When nested object read event, go to end
@@ -2127,8 +2137,11 @@ st_menu_driver(struct ST_MENU *menu, int c, bool alt, MEVENT *mevent)
 	 * is pressed, we don't know a related object, but
 	 * we can reset selected_command variable.
 	 */
-	if (mevent->bstate & BUTTON1_PRESSED)
+	if (c != KEY_MOUSE || mevent->bstate & BUTTON1_PRESSED)
+	{
 		selected_command = NULL;
+		command_was_activated = false;
+	}
 
 	/*
 	 * We would to close pulldown menus on F10 key - similar behave
@@ -3085,3 +3098,40 @@ st_menu_set_ref_option(struct ST_MENU *menu,
 
 	return false;
 }
+
+/*
+ * Set shortcut for option specified by code
+ */
+bool
+st_menu_set_shortcut(struct ST_MENU *menu, int code, char *shortcut)
+{
+	ST_MENU_ITEM *menu_items = menu->menu_items;
+	int		i = 0;
+
+	while (menu_items->text)
+	{
+		if (menu_items->code == code)
+		{
+			menu_items->shortcut = shortcut;
+			return true;
+		}
+
+		if (menu->submenus[i])
+			if (st_menu_set_shortcut(menu->submenus[i], code, shortcut))
+				return true;
+
+		menu_items += 1;
+		i += 1;
+	}
+
+	return false;
+}
+
+/*
+ * Returns type of focus of menu
+ */
+int
+st_menu_get_focus(struct ST_MENU *menu)
+{
+	return menu->focus;
+}
