@@ -147,7 +147,7 @@ isBottomLeftChar(char *str)
  * detect different faces of headline in extended mode
  */
 bool
-is_expanded_header(Options *opts, char *str, int *ei_minx, int *ei_maxx)
+is_expanded_header(char *str, int *ei_minx, int *ei_maxx)
 {
 	int		pos = 0;
 
@@ -191,7 +191,7 @@ is_expanded_header(Options *opts, char *str, int *ei_minx, int *ei_maxx)
 		while (*str != ']' && *str != '\0')
 		{
 			pos += 1;
-			str += opts->force8bit ? 1 : utf8charlen(*str);
+			str += charlen(str);
 		}
 
 		*ei_maxx = pos - 1;
@@ -480,7 +480,7 @@ endline_exit:
  * Copy trimmed string
  */
 static void
-strncpytrim(Options *opts, char *dest, const char *src,
+strncpytrim(char *dest, const char *src,
 			size_t ndest, size_t nsrc)
 {
 	const char *endptr;
@@ -510,7 +510,7 @@ strncpytrim(Options *opts, char *dest, const char *src,
 		if (*src == '\0')
 			break;
 
-		clen = (size_t) opts->force8bit ? 1 : utf8charlen(*src);
+		clen = (size_t) charlen(src);
 		if (clen <= ndest && clen <= nsrc)
 		{
 			size_t		i;
@@ -593,6 +593,9 @@ readfile(Options *opts, DataDesc *desc, StateData *state)
 		desc->filename[64] = '\0';
 	}
 
+	if (!state->fp)
+		return false;
+
 	clearerr(state->fp);
 
 	/* detection truncating */
@@ -639,9 +642,16 @@ readfile(Options *opts, DataDesc *desc, StateData *state)
 			break;
 		}
 
-		clen = utf_string_dsplen(line, read);
+		/* In query stream node exit when you find row with only GS - Group Separator */
+		if (opts->querystream && read == 1)
+		{
+			if (*line == 0x1D)
+				break;
+		}
 
-		if (rows->nrows == 1000)
+		clen = use_utf8 ? utf_string_dsplen(line, read) : read;
+
+		if (rows->nrows == LINEBUFFER_LINES)
 		{
 			LineBuffer *newrows = smalloc(sizeof(LineBuffer));
 
@@ -652,24 +662,34 @@ readfile(Options *opts, DataDesc *desc, StateData *state)
 
 		rows->rows[rows->nrows++] = line;
 
+		/*
+		 * The input file is not an table
+		 */
+		if (opts->querystream)
+		{
+			nrows += 1;
+			desc->last_row = nrows - 1;
+			goto next_row;
+		}
+
 		/* save possible table name */
 		if (nrows == 0 && !isTopLeftChar(line))
 		{
-			strncpytrim(opts, desc->title, line, 63, read);
+			strncpytrim(desc->title, line, 63, read);
 			desc->title_rows = 1;
 		}
 
 		if (desc->border_head_row == -1 && desc->border_top_row == -1 && isTopLeftChar(line))
 		{
 			desc->border_top_row = nrows;
-			desc->is_expanded_mode = is_expanded_header(opts, line, NULL, NULL);
+			desc->is_expanded_mode = is_expanded_header(line, NULL, NULL);
 		}
 		else if (desc->border_head_row == -1 && isHeadLeftChar(line))
 		{
 			desc->border_head_row = nrows;
 
 			if (!desc->is_expanded_mode)
-				desc->is_expanded_mode = is_expanded_header(opts, line, NULL, NULL);
+				desc->is_expanded_mode = is_expanded_header(line, NULL, NULL);
 
 			/* title surely doesn't it there */
 			if ((!desc->is_expanded_mode && nrows == 1) ||
@@ -761,6 +781,10 @@ next_row:
 		desc->headline = desc->rows.rows[desc->border_head_row];
 		desc->headline_size = strlen(desc->headline);
 
+		/*
+		 * fallback, but can be fixed later, when border_type
+		 * will be known.
+		 */
 		if (desc->last_data_row == -1)
 			desc->last_data_row = desc->last_row - 1;
 
@@ -804,7 +828,7 @@ broken_format:
  * Translate from UTF8 to semantic characters.
  */
 bool
-translate_headline(Options *opts, DataDesc *desc)
+translate_headline(DataDesc *desc)
 {
 	char   *srcptr;
 	char   *destptr;
@@ -812,7 +836,6 @@ translate_headline(Options *opts, DataDesc *desc)
 	bool	broken_format = false;
 	int		processed_chars = 0;
 	bool	is_expanded_info = false;
-	bool	force8bit = opts->force8bit;
 
 	srcptr = desc->headline;
 	destptr = smalloc(desc->headline_size + 2);
@@ -852,7 +875,7 @@ translate_headline(Options *opts, DataDesc *desc)
 			desc->expanded_info_minx = processed_chars;
 
 			*destptr++ = 'd';
-			srcptr += force8bit ? 1 : utf8charlen(*srcptr);
+			srcptr += charlen(srcptr);
 		}
 		else if (is_expanded_info)
 		{
@@ -861,7 +884,7 @@ translate_headline(Options *opts, DataDesc *desc)
 				is_expanded_info = false;
 			}
 			*destptr++ = 'd';
-			srcptr += force8bit ? 1 : utf8charlen(*srcptr);
+			srcptr += charlen(srcptr);
 		}
 		else if (strncmp(srcptr, "\342\224\214", 3) == 0 || /* ┌ */
 				  strncmp(srcptr, "\342\225\224", 3) == 0)   /* ╔ */
@@ -1099,8 +1122,8 @@ translate_headline(Options *opts, DataDesc *desc)
 				}
 				else
 				{
-					nextchar = namesline + (opts->force8bit ? 1 : utf8charlen(*namesline));
-					display_width = opts->force8bit ? 1 : utf_dsplen(namesline);
+					nextchar = namesline + charlen(namesline);
+					display_width = dsplen(namesline);
 				}
 			}
 			else
@@ -1185,7 +1208,6 @@ cut_text(char *str,
 		 int xmin,
 		 int xmax,
 		 bool border0,
-		 bool force8bit,
 		 char **result)
 {
 #define TEXT_STACK_BUFFER_SIZE		1024
@@ -1195,12 +1217,12 @@ cut_text(char *str,
 		char	   *_str = NULL;
 		char	   *after_last_nospc = NULL;
 		int			pos = 0;
-		int			charlen;
+		int			chrlen;
 		bool		skip_left_spaces = true;
 
 		while (*str)
 		{
-			charlen = utf8charlen(*str);
+			chrlen = charlen(str);
 
 			if (pos > xmin || (border0 && pos >= xmin))
 			{
@@ -1220,10 +1242,10 @@ cut_text(char *str,
 			}
 
 			if (*str != ' ')
-				after_last_nospc = str + charlen;
+				after_last_nospc = str + chrlen;
 
 			pos += utf_dsplen(str);
-			str += charlen;
+			str += chrlen;
 
 			if (pos >= xmax)
 				break;
@@ -1241,7 +1263,7 @@ cut_text(char *str,
 			if (!cstr)
 				leave("out of memory");
 
-			if (force8bit)
+			if (!use_utf8)
 			{
 				*result = cstr;
 				return true;
@@ -1327,7 +1349,7 @@ cut_numeric_value(char *str, int xmin, int xmax, double *d, bool border0, bool *
 
 		while (*str)
 		{
-			int		charlen = utf8charlen(*str);
+			int		chrlen = charlen(str);
 
 			if (x > xmin || (border0 && x >= xmin))
 			{
@@ -1358,16 +1380,16 @@ cut_numeric_value(char *str, int xmin, int xmax, double *d, bool border0, bool *
 						while (*str)
 						{
 							if (*str != ' ')
-								after_last_nospace = str + charlen;
+								after_last_nospace = str + chrlen;
 
-							x += utf_dsplen(str);
-							str += charlen;
+							x += dsplen(str);
+							str += chrlen;
 
 							if (x >= xmax)
 								break;
 
 							if (*str)
-								charlen = utf8charlen(*str);
+								chrlen = charlen(str);
 						}
 
 						len = after_last_nospace - saved_str;
@@ -1397,7 +1419,7 @@ cut_numeric_value(char *str, int xmin, int xmax, double *d, bool border0, bool *
 					only_digits = true;
 				}
 
-				memcpy(buffptr, str, charlen);
+				memcpy(buffptr, str, chrlen);
 
 				/* trim from right */
 				if (c != ' ')
@@ -1405,7 +1427,7 @@ cut_numeric_value(char *str, int xmin, int xmax, double *d, bool border0, bool *
 					bool	only_digits_prev = only_digits;
 					bool	only_digits_with_point_prev = only_digits_with_point;
 
-					after_last_nospace = buffptr + charlen;
+					after_last_nospace = buffptr + chrlen;
 					if (after_last_nospace - buffer > (BUFFER_MAX_SIZE - 1))
 					{
 						/* too long string - should not be translated to number */
@@ -1436,11 +1458,11 @@ cut_numeric_value(char *str, int xmin, int xmax, double *d, bool border0, bool *
 						first_nospace_nodigit = buffptr;
 					}
 				}
-				buffptr += charlen;
+				buffptr += chrlen;
 			}
 
-			x += utf_dsplen(str);
-			str += charlen;
+			x += dsplen(str);
+			str += chrlen;
 
 			if (x >= xmax)
 				break;
@@ -1492,25 +1514,100 @@ cut_numeric_value(char *str, int xmin, int xmax, double *d, bool border0, bool *
 	return false;
 }
 
+/*
+ * Try to detect multiline rows.
+ */
+void
+multilines_detection(DataDesc *desc)
+{
+	LineBufferIter	lbi;
+	LineBufferMark	lbm;
+
+	bool		border0 = (desc->border_type == 0);
+	bool		border1 = (desc->border_type == 1);
+	bool		border2 = (desc->border_type == 2);
+
+	bool		has_multilines = false;
+
+	if (desc->multilines_already_tested)
+		return;
+
+	init_lbi_ddesc(&lbi, desc, 0);
+
+	while (lbi_set_mark_next(&lbi, &lbm))
+	{
+		char	   *str;
+		int			lineno;
+		int			pos = 0;
+		bool		found_continuation_symbol = false;
+
+		(void) lbm_get_line(&lbm, &str, NULL, &lineno);
+
+		if (lineno < desc->first_data_row || lineno > desc->last_data_row)
+			continue;
+
+		/*
+		 * This implementation doesn't support old-ascii format
+		 */
+		while (pos < desc->headline_char_size)
+		{
+			if (border0)
+			{
+				if (pos + 1 == desc->headline_char_size)
+				{
+					char	*sym;
+
+					sym = str + charlen(str);
+					if (*sym != '\0')
+						found_continuation_symbol = is_line_continuation_char(sym, desc);
+				}
+				else if (desc->headline_transl[pos] == 'I')
+					found_continuation_symbol = is_line_continuation_char(str, desc);
+			}
+			else if (border1)
+			{
+				if ((pos + 1 < desc->headline_char_size && desc->headline_transl[pos + 1] == 'I') ||
+					  (pos + 1 == desc->headline_char_size))
+					found_continuation_symbol = is_line_continuation_char(str, desc);
+			}
+			else if (border2)
+			{
+				if ((pos + 1 < desc->headline_char_size) &&
+					  (desc->headline_transl[pos + 1] == 'I' || desc->headline_transl[pos + 1] == 'R'))
+					found_continuation_symbol = is_line_continuation_char(str, desc);
+			}
+
+			if (found_continuation_symbol)
+			{
+				lbm_xor_mask(&lbm, LINEINFO_CONTINUATION);
+				has_multilines = true;
+				break;
+			}
+
+			pos += dsplen(str);
+			str += charlen(str);
+		}
+	}
+
+	desc->multilines_already_tested = true;
+	desc->has_multilines = has_multilines;
+}
 
 /*
  * Prepare order map - it is used for printing data in different than
  * original order. "sbcn" - sort by column number
  */
 void
-update_order_map(Options *opts, ScrDesc *scrdesc, DataDesc *desc, int sbcn, bool desc_sort)
+update_order_map(ScrDesc *scrdesc, DataDesc *desc, int sbcn, bool desc_sort)
 {
 	LineBuffer	   *lnb = &desc->rows;
 	char		   *nullstr = NULL;
 	int				xmin, xmax;
 	int				lineno = 0;
 	bool			continual_line = false;
-	bool			has_multilines = false;
 	bool			isnull;
 	bool			detect_string_column = false;
 	bool			border0 = (desc->border_type == 0);
-	bool			border1 = (desc->border_type == 1);
-	bool			border2 = (desc->border_type == 2);
 	SortData	   *sortbuf;
 	int				sortbuf_pos = 0;
 	int			i;
@@ -1520,82 +1617,18 @@ update_order_map(Options *opts, ScrDesc *scrdesc, DataDesc *desc, int sbcn, bool
 
 	sortbuf = smalloc(desc->total_rows * sizeof(SortData));
 
-	/* first time we should to detect multilines */
-	if (!desc->multilines_already_tested)
-	{
-		desc->multilines_already_tested = true;
-
-		lnb = &desc->rows;
-		lineno = 0;
-
-		while (lnb)
-		{
-			for (i = 0; i < lnb->nrows; i++)
-			{
-				if (lineno >= desc->first_data_row && lineno <= desc->last_data_row)
-				{
-					char   *str = lnb->rows[i];
-					bool	found_continuation_symbol = false;
-					int		j = 0;
-
-					while (j < desc->headline_char_size)
-					{
-						if (border0)
-						{
-							/* border 0, last continuation symbol is after headline */
-							if (j + 1 == desc->headline_char_size)
-							{
-								char	*sym;
-
-								sym = str + (opts->force8bit ? 1 : utf8charlen(*str));
-								if (*sym != '\0')
-									found_continuation_symbol = is_line_continuation_char(sym, desc);
-							}
-							else if (desc->headline_transl[j] == 'I')
-								found_continuation_symbol = is_line_continuation_char(str, desc);
-						}
-						else if (border1)
-						{
-							if ((j + 1 < desc->headline_char_size && desc->headline_transl[j + 1] == 'I') ||
-									  (j + 1 == desc->headline_char_size))
-								found_continuation_symbol = is_line_continuation_char(str, desc);
-						}
-						else if (border2)
-						{
-							if ((j + 1 < desc->headline_char_size) &&
-									(desc->headline_transl[j + 1] == 'I' || desc->headline_transl[j + 1] == 'R'))
-								found_continuation_symbol = is_line_continuation_char(str, desc);
-						}
-
-						if (found_continuation_symbol)
-							break;
-
-						j += opts->force8bit ? 1 : utf_dsplen(str);
-						str += opts->force8bit ? 1 : utf8charlen(*str);
-					}
-
-					if (found_continuation_symbol)
-					{
-						if (lnb->lineinfo == NULL)
-							lnb->lineinfo = smalloc(1000 * sizeof(LineInfo));
-
-						lnb->lineinfo[i].mask ^= LINEINFO_CONTINUATION;
-						has_multilines = true;
-					}
-				}
-
-				lineno += 1;
-			}
-			lnb = lnb->next;
-		}
-	}
+	/* multilines should be detected first */
+	multilines_detection(desc);
 
 	lnb = &desc->rows;
 	lineno = 0;
 	sortbuf_pos = 0;
 
 	if (!desc->order_map)
+	{
 		desc->order_map = smalloc(desc->total_rows * sizeof(MappedLine));
+		desc->order_map_items = desc->total_rows;
+	}
 
 	/*
 	 * There are two possible sorting methods: numeric or string.
@@ -1639,7 +1672,7 @@ update_order_map(Options *opts, ScrDesc *scrdesc, DataDesc *desc, int sbcn, bool
 					}
 				}
 
-				if (has_multilines)
+				if (desc->has_multilines)
 				{
 					continual_line = (lnb->lineinfo &&
 									  (lnb->lineinfo[i].mask & LINEINFO_CONTINUATION));
@@ -1677,13 +1710,13 @@ sort_by_string:
 						sortbuf[sortbuf_pos].lnb_row = i;
 						sortbuf[sortbuf_pos].d = 0.0;
 
-						if (cut_text(lnb->rows[i], xmin, xmax, border0, opts->force8bit, &sortbuf[sortbuf_pos].strxfrm))
+						if (cut_text(lnb->rows[i], xmin, xmax, border0, &sortbuf[sortbuf_pos].strxfrm))
 							sortbuf[sortbuf_pos++].info = INFO_STRXFRM;
 						else
 							sortbuf[sortbuf_pos++].info = INFO_UNKNOWN;		/* empty string */
 					}
 
-					if (has_multilines)
+					if (desc->has_multilines)
 					{
 						continual_line =  (lnb->lineinfo &&
 										   (lnb->lineinfo[i].mask & LINEINFO_CONTINUATION));
@@ -1713,7 +1746,7 @@ sort_by_string:
 		lineno += 1;
 
 		/* assign other continual lines */
-		if (has_multilines)
+		if (desc->has_multilines)
 		{
 			int		lnb_row;
 			bool	continual = false;
