@@ -74,12 +74,20 @@ initialize_color_pairs(int theme, bool bold_labels, bool bold_cursor)
 	attr_t labels_attr = bold_labels ? A_BOLD : 0;
 	attr_t cursor_attr = bold_cursor ? A_BOLD : 0;
 
+	memset(theme_attrs, 0, sizeof(theme_attrs));
+
 	init_pair(21, COLOR_WHITE, COLOR_BLACK);		/* Fx keys */
 
 	set_colour(26, COLOR_WHITE, COLOR_RED, true, 0);		/* error */
 	set_colour(27, COLOR_BLACK, COLOR_WHITE, false, 0);		/* input */
 
-	memset(theme_attrs, 0, sizeof(theme_attrs));
+	set_colour(30, COLOR_WHITE, COLOR_BLACK, false, A_REVERSE);	/* scrollbar arrows */
+	set_colour(31, COLOR_WHITE, COLOR_BLACK, true, 0);			/* scrollbar background */
+	set_colour(32, COLOR_BLACK, COLOR_BLUE, false, A_REVERSE);	/* scrollbar slider */
+	set_colour(33, COLOR_WHITE, COLOR_BLACK, true, A_REVERSE);	/* scrollbar active slider */
+
+	set_colour(34, COLOR_CYAN, COLOR_BLACK, true, A_REVERSE);			/* top bar colors */
+	set_colour(35, COLOR_WHITE, COLOR_BLACK, true, A_REVERSE);			/* top bar colors */
 
 	switch (theme)
 	{
@@ -114,6 +122,12 @@ initialize_color_pairs(int theme, bool bold_labels, bool bold_cursor)
 			set_colour(25, COLOR_BLACK, COLOR_WHITE, false, cursor_attr);/* vertical cursor pattern line */
 			set_colour(27, -1, -1, false, 0);		/* input */
 			set_colour(28, COLOR_BLACK, COLOR_WHITE, false, 0);			/* color of bookmark lines for line art */
+			set_colour(30, -1, -1, false, 0);							/* scrollbar arrows */
+			set_colour(31, -1, -1, true, A_DIM);						/* scrollbar background */
+			set_colour(32, -1, -1, false,  0);							/* scrollbar slider */
+			set_colour(33, -1, -1, false, A_REVERSE);					/* scrollbar active slider */
+			set_colour(34, COLOR_WHITE, COLOR_BLACK, false, A_DIM | A_BOLD | A_REVERSE);			/* selected data */
+			set_colour(35, COLOR_BLACK, COLOR_WHITE, true, A_BOLD | A_REVERSE);	/* cursor in selected area */
 			break;
 
 		case 1:
@@ -146,6 +160,10 @@ initialize_color_pairs(int theme, bool bold_labels, bool bold_cursor)
 			set_colour(25, COLOR_GREEN, COLOR_WHITE, true, A_REVERSE);
 			set_colour(26, COLOR_WHITE, COLOR_RED, true, 0);
 			set_colour(28, COLOR_WHITE, COLOR_RED, true, 0);
+			set_colour(30, COLOR_WHITE, COLOR_BLUE, false, 0);
+			set_colour(31, COLOR_CYAN, COLOR_BLUE, false, 0);
+			set_colour(32, COLOR_WHITE, COLOR_BLUE, false, A_REVERSE);
+			set_colour(33, COLOR_WHITE, COLOR_BLUE, true, A_REVERSE);
 			break;
 
 		case 2:
@@ -178,6 +196,12 @@ initialize_color_pairs(int theme, bool bold_labels, bool bold_cursor)
 			set_colour(25, COLOR_GREEN, COLOR_WHITE, true, A_REVERSE);
 			set_colour(26, COLOR_YELLOW, COLOR_RED, true,0);
 			set_colour(28, COLOR_WHITE, COLOR_MAGENTA, true,0);
+			set_colour(30, COLOR_YELLOW, COLOR_WHITE, true, 0);
+			set_colour(31, COLOR_WHITE, COLOR_WHITE, false, 0);
+			set_colour(32, COLOR_YELLOW, COLOR_WHITE, true, 0);
+			set_colour(33, COLOR_YELLOW, COLOR_WHITE, true, A_BOLD);
+			set_colour(34, COLOR_CYAN, COLOR_BLACK, true, A_REVERSE);
+			set_colour(35, COLOR_WHITE, COLOR_BLACK, true, A_REVERSE | cursor_attr );
 			break;
 
 		case 3:
@@ -209,6 +233,12 @@ initialize_color_pairs(int theme, bool bold_labels, bool bold_cursor)
 			set_colour(24, COLOR_GREEN, COLOR_BLACK, true, A_REVERSE | cursor_attr);
 			set_colour(25, COLOR_GREEN, COLOR_BLACK, true, A_REVERSE);
 			set_colour(28, COLOR_WHITE, COLOR_RED, true, 0);
+			set_colour(30, COLOR_WHITE, COLOR_BLACK, true, A_REVERSE);
+			set_colour(31, COLOR_WHITE, COLOR_BLACK, true, 0);
+			set_colour(32, COLOR_BLACK, COLOR_BLUE, false, A_REVERSE);
+			set_colour(33, COLOR_WHITE, COLOR_BLACK, true, A_REVERSE);
+			set_colour(34, COLOR_CYAN, COLOR_BLACK, true, A_REVERSE);
+			set_colour(35, COLOR_WHITE, COLOR_BLACK, true, A_REVERSE | cursor_attr );
 			break;
 
 		case 4:
@@ -217,7 +247,7 @@ initialize_color_pairs(int theme, bool bold_labels, bool bold_cursor)
 
 			set_colour(2, COLOR_BLACK, COLOR_CYAN, false, 0);
 			set_colour(3, COLOR_BLACK, COLOR_WHITE, false, 0);
-			set_colour(4, COLOR_BLACK, COLOR_WHITE, true, labels_attr);
+			set_colour(4, COLOR_BLACK, COLOR_WHITE, false, A_BOLD);
 			set_colour(5, COLOR_WHITE, COLOR_BLUE, true, cursor_attr);
 			set_colour(6, COLOR_WHITE, COLOR_BLUE, true, cursor_attr);
 			set_colour(7, COLOR_BLACK, COLOR_CYAN, false, 0);
@@ -334,6 +364,12 @@ initialize_color_pairs(int theme, bool bold_labels, bool bold_cursor)
 			set_colour(25, COLOR_CYAN, COLOR_GREEN, true, A_REVERSE);
 			set_colour(27, COLOR_WHITE, COLOR_BLACK, true, 0);		/* input */
 			set_colour(28, COLOR_WHITE, COLOR_RED, true, 0);
+			set_colour(30, COLOR_GREEN, COLOR_BLACK, true, 0);
+			set_colour(31, COLOR_GREEN, COLOR_BLACK, false, 0);
+			set_colour(32, COLOR_GREEN, COLOR_BLACK, false, A_REVERSE);
+			set_colour(33, COLOR_GREEN, COLOR_BLACK, true, A_REVERSE);
+			set_colour(34, COLOR_GREEN, COLOR_BLACK, true, A_DIM | A_REVERSE);
+			set_colour(35, COLOR_GREEN, COLOR_BLACK, true, A_REVERSE | cursor_attr );
 			break;
 
 		case 8:
@@ -366,6 +402,10 @@ initialize_color_pairs(int theme, bool bold_labels, bool bold_cursor)
 			set_colour(25, COLOR_GREEN, COLOR_CYAN, true, A_REVERSE);
 			set_colour(27, COLOR_WHITE, COLOR_BLACK, true, A_REVERSE);		/* input */
 			set_colour(28, COLOR_WHITE, COLOR_RED, true, 0);
+			set_colour(30, COLOR_WHITE, COLOR_BLACK, true, A_REVERSE);	/* scrollbar arrows */
+			set_colour(31, COLOR_WHITE, COLOR_WHITE, false, 0);			/* scrollbar background */
+			set_colour(32, COLOR_WHITE, COLOR_WHITE, true, A_REVERSE);	/* scrollbar slider */
+			set_colour(33, COLOR_BLACK, COLOR_BLACK, false, A_REVERSE);	/* scrollbar active slider */
 			break;
 
 		case 9:
@@ -379,7 +419,7 @@ initialize_color_pairs(int theme, bool bold_labels, bool bold_cursor)
 			set_colour(6, COLOR_WHITE, COLOR_CYAN, true, cursor_attr);
 			set_colour(7, COLOR_BLACK, COLOR_WHITE, false, 0);
 			set_colour(8, COLOR_WHITE, COLOR_BLUE, false, 0);
-			set_colour(9, COLOR_WHITE, COLOR_BLUE, false, 0);
+			set_colour(9, COLOR_GREEN, COLOR_BLUE, true, 0);
 			set_colour(10, COLOR_WHITE, COLOR_CYAN, true, cursor_attr);
 			set_colour(11, COLOR_WHITE, COLOR_CYAN, false, 0);
 			set_colour(12, COLOR_WHITE, COLOR_CYAN, true, 0);
@@ -397,6 +437,10 @@ initialize_color_pairs(int theme, bool bold_labels, bool bold_cursor)
 			set_colour(24, COLOR_GREEN, COLOR_BLACK, true, A_REVERSE | cursor_attr);
 			set_colour(25, COLOR_GREEN, COLOR_WHITE, true, A_REVERSE);
 			set_colour(28, COLOR_WHITE, COLOR_RED, true, 0);
+			set_colour(30, COLOR_WHITE, COLOR_BLUE, true, 0);
+			set_colour(31, COLOR_WHITE, COLOR_BLUE, false, 0);
+			set_colour(32, COLOR_WHITE, COLOR_WHITE, false, A_REVERSE);
+			set_colour(33, COLOR_WHITE, COLOR_WHITE, true, A_REVERSE);
 			break;
 
 		case 10:
@@ -429,6 +473,12 @@ initialize_color_pairs(int theme, bool bold_labels, bool bold_cursor)
 			set_colour(25, COLOR_GREEN, COLOR_BLUE, true, A_REVERSE);
 			set_colour(27, COLOR_BLACK, COLOR_CYAN, false, 0);		/* input */
 			set_colour(28, COLOR_WHITE, COLOR_RED, true, 0);
+			set_colour(30, COLOR_BLUE, COLOR_CYAN, false, 0);
+			set_colour(31, COLOR_BLACK, COLOR_CYAN, true, 0);
+			set_colour(32, COLOR_CYAN, COLOR_BLACK, true, A_REVERSE);
+			set_colour(33, COLOR_WHITE, COLOR_BLACK, true, A_REVERSE);
+			set_colour(34, COLOR_CYAN, COLOR_BLACK, true, A_DIM | A_REVERSE);
+			set_colour(35, COLOR_GREEN, COLOR_BLACK, false, A_REVERSE | cursor_attr );
 			break;
 
 		case 11:
@@ -460,6 +510,12 @@ initialize_color_pairs(int theme, bool bold_labels, bool bold_cursor)
 			set_colour(25, COLOR_GREEN, COLOR_CYAN, true, A_REVERSE);
 			set_colour(27, COLOR_WHITE, COLOR_BLACK, true, 0);		/* input */
 			set_colour(28, COLOR_WHITE, COLOR_RED, true, 0);
+			set_colour(30, COLOR_BLACK, COLOR_WHITE, false, 0);
+			set_colour(31, COLOR_CYAN, COLOR_BLUE, false, 0);
+			set_colour(32, COLOR_WHITE, COLOR_WHITE, false, A_REVERSE);
+			set_colour(33, COLOR_WHITE, COLOR_WHITE, true, A_REVERSE);
+			set_colour(34, COLOR_CYAN, COLOR_BLACK, true, A_REVERSE);
+			set_colour(35, COLOR_CYAN, COLOR_BLUE, true, cursor_attr );
 			break;
 
 		case 12:
@@ -491,6 +547,12 @@ initialize_color_pairs(int theme, bool bold_labels, bool bold_cursor)
 			set_colour(24, COLOR_GREEN, COLOR_BLACK, true, A_REVERSE | cursor_attr);
 			set_colour(25, COLOR_GREEN, COLOR_BLUE, true, A_REVERSE);
 			set_colour(28, COLOR_WHITE, COLOR_RED, true, 0);
+			set_colour(30, COLOR_BLUE, COLOR_CYAN, false, 0);
+			set_colour(31, COLOR_CYAN, COLOR_BLUE, false, 0);
+			set_colour(32, COLOR_BLUE, COLOR_CYAN, false, 0);
+			set_colour(33, COLOR_CYAN, COLOR_BLUE, true, A_REVERSE);
+			set_colour(34, COLOR_CYAN, COLOR_BLACK, true, A_REVERSE);
+			set_colour(35, COLOR_CYAN, COLOR_BLACK, true, A_DIM | A_REVERSE | cursor_attr );
 			break;
 
 		case 13:
@@ -522,7 +584,14 @@ initialize_color_pairs(int theme, bool bold_labels, bool bold_cursor)
 			set_colour(25, COLOR_GREEN, COLOR_WHITE, true, A_REVERSE);
 			set_colour(27, COLOR_BLACK, COLOR_CYAN, false, 0);		/* input */
 			set_colour(28, COLOR_WHITE, COLOR_RED, true, 0);
+			set_colour(30, COLOR_WHITE, COLOR_BLUE, true, 0);
+			set_colour(31, COLOR_CYAN, COLOR_BLUE, false, 0);
+			set_colour(32, COLOR_WHITE, COLOR_BLUE, false, A_REVERSE);
+			set_colour(33, COLOR_WHITE, COLOR_BLACK, true, A_REVERSE);
+			set_colour(34, COLOR_WHITE, COLOR_MAGENTA, true, 0);
+			set_colour(35, COLOR_MAGENTA, COLOR_BLACK, true, A_REVERSE | cursor_attr );
 			break;
+
 		case 14:
 			/* DBase retro magenta */
 			init_pair(1, COLOR_WHITE, COLOR_BLUE);
@@ -553,6 +622,10 @@ initialize_color_pairs(int theme, bool bold_labels, bool bold_cursor)
 			set_colour(25, COLOR_GREEN, COLOR_WHITE, true, A_REVERSE);
 			set_colour(27, COLOR_WHITE, COLOR_BLUE, true, 0);		/* input */
 			set_colour(28, COLOR_WHITE, COLOR_RED, true, 0);
+			set_colour(30, COLOR_WHITE, COLOR_BLUE, true, 0);
+			set_colour(31, COLOR_CYAN, COLOR_BLUE, false, 0);
+			set_colour(32, COLOR_WHITE, COLOR_BLUE, false, A_REVERSE);
+			set_colour(33, COLOR_MAGENTA, COLOR_BLACK, true, A_REVERSE);
 			break;
 
 		case 15:
@@ -584,6 +657,12 @@ initialize_color_pairs(int theme, bool bold_labels, bool bold_cursor)
 			set_colour(25, COLOR_RED, COLOR_BLACK, true, A_REVERSE);
 			set_colour(27, COLOR_WHITE, COLOR_BLACK, true, A_REVERSE);		/* input */
 			set_colour(28, COLOR_YELLOW, COLOR_RED, true, A_REVERSE | A_BOLD);
+			set_colour(30, COLOR_BLACK, COLOR_WHITE, false, 0);
+			set_colour(31, COLOR_BLACK, COLOR_WHITE, true, 0);
+			set_colour(32, COLOR_BLACK, COLOR_BLACK, true, A_REVERSE | A_DIM);
+			set_colour(33, COLOR_RED, COLOR_BLACK, false, A_REVERSE);
+			set_colour(34, COLOR_YELLOW, COLOR_BLACK, true, A_REVERSE);
+			set_colour(35, COLOR_RED, COLOR_BLACK, true, A_REVERSE | cursor_attr );
 			break;
 
 		case 16:
@@ -617,6 +696,12 @@ initialize_color_pairs(int theme, bool bold_labels, bool bold_cursor)
 			set_colour(25, -1, -1, true, A_REVERSE);
 			set_colour(27, -1, -1, false, 0);		/* input */
 			set_colour(28, -1, -1, false, A_UNDERLINE);
+			set_colour(30, -1, -1, true, 0);
+			set_colour(31, -1, -1, false, A_DIM);
+			set_colour(32, -1, -1, false, A_REVERSE);
+			set_colour(33, -1, -1, true, A_REVERSE);
+			set_colour(34, -1, -1, false, A_DIM | A_REVERSE);
+			set_colour(35, -1, -1, false, A_REVERSE | cursor_attr | A_UNDERLINE);
 			break;
 
 		case 17:
@@ -731,7 +816,6 @@ initialize_color_pairs(int theme, bool bold_labels, bool bold_cursor)
 			init_color_rgb_ff(214, 0xd0, 0xcf, 0xa0); /* mark line cursor */
 			init_color_rgb_ff(215, 0xff, 0xff, 0xff); /* mark fg - white */
 
-
 			init_pair(1, 204, 200);
 
 			init_pair(2, 203, 202);
@@ -763,6 +847,9 @@ initialize_color_pairs(int theme, bool bold_labels, bool bold_cursor)
 			theme_attrs[5] = cursor_attr;
 			theme_attrs[6] = cursor_attr;
 			theme_attrs[10] = cursor_attr;
+
+			set_colour(34, 204, 201, false, A_REVERSE);
+			set_colour(35, 201, 202, false, cursor_attr);
 			break;
 
 		case 20:
@@ -822,6 +909,9 @@ initialize_color_pairs(int theme, bool bold_labels, bool bold_cursor)
 			theme_attrs[5] = cursor_attr;
 			theme_attrs[6] = cursor_attr;
 			theme_attrs[10] = cursor_attr;
+
+			set_colour(34, 203, 206, false, 0);
+			set_colour(35, 213, 203, false, cursor_attr);
 			break;
 
 		case 21:
@@ -883,10 +973,13 @@ initialize_color_pairs(int theme, bool bold_labels, bool bold_cursor)
 			theme_attrs[5] = cursor_attr;
 			theme_attrs[6] = cursor_attr;
 			theme_attrs[10] = cursor_attr;
+
+			set_colour(34, 210, 202, false, 0);
+			set_colour(35, 213, 203, false, cursor_attr);
 			break;
 
 		case 22:
-			/* mc theme */
+			/* Relation pipes theme */
 			init_pair(1, COLOR_GREEN, COLOR_BLACK);
 
 			set_colour(2, COLOR_RED, COLOR_BLACK, true, 0);
@@ -915,6 +1008,77 @@ initialize_color_pairs(int theme, bool bold_labels, bool bold_cursor)
 			set_colour(25, COLOR_GREEN, COLOR_WHITE, true, A_REVERSE);
 			set_colour(26, COLOR_WHITE, COLOR_RED, true, 0);
 			set_colour(28, COLOR_GREEN, COLOR_RED, true, 0);
+			set_colour(30, COLOR_WHITE, COLOR_BLACK, false, A_REVERSE);
+			set_colour(31, COLOR_BLACK, COLOR_BLACK, true, 0);
+			set_colour(32, COLOR_WHITE, COLOR_BLUE, false, A_REVERSE);
+			set_colour(33, COLOR_WHITE, COLOR_BLACK, true, A_REVERSE);
+			break;
+
+		case 23:
+			/* PaperColour theme */
+			init_color_rgb_ff(200, 0xee, 0xee, 0xee); /* background */
+			init_color_rgb_ff(201, 0x09, 0x09, 0x09); /* foreground */
+			init_color_rgb_ff(202, 0xd0, 0xd0, 0xd0); /* modeline bg */
+			init_color_rgb_ff(203, 0x09, 0x09, 0x09); /* modeline fg */
+			init_color_rgb_ff(204, 0x00, 0x87, 0xaf); /* table decor */
+			init_color_rgb_ff(205, 0x00, 0x5f, 0x87); /* cursor bg */
+			init_color_rgb_ff(213, 0xee, 0xee, 0xee); /* cursor fg */
+			init_color_rgb_ff(206, 0x44, 0x44, 0x44); /* footer */
+			init_color_rgb_ff(207, 0xee, 0xee, 0xee); /* lineno bg */
+
+			init_color_rgb_ff(210, 0xd7, 0x00, 0x87); /* labels */
+			init_color_rgb_ff(211, 0Xdf, 0x00, 0x00); /* bookmark - bg */
+			init_color_rgb_ff(212, 0xff, 0xff, 0xf0); /* bookmark - fg */
+			init_color_rgb_ff(215, 0xff, 0xff, 0x87); /* mark bg */
+			init_color_rgb_ff(214, 0xff, 0xff, 0x87); /* marked line bg */
+			init_color_rgb_ff(216, 0x25, 0x25, 0x25);
+
+			init_color_rgb_ff(217, 0x00, 0x5f, 0x40); /* vertical marked cursor bg */
+			init_color_rgb_ff(218, 0xff, 0xff, 0xff);
+			init_color_rgb_ff(219, 0x5f, 0x45, 0xe5); 
+			init_color_rgb_ff(199, 0x00, 0x5f, 0xd0); /* cross cursor */
+
+			init_color(240, 40, 50, 200);
+
+			init_pair(1, 204, 200);
+			init_pair(2, 203, 202);
+			init_pair(3, 201, 200);
+			init_pair(4, 210, 200);
+			init_pair(5, 213, 205);
+			init_pair(6, 213, 205);
+			init_pair(7, 203, 202);
+			init_pair(8, 17, 200);
+			init_pair(9, 206, 200);
+			init_pair(10, 213, 205);
+			init_pair(11, 204, 205);
+			init_pair(13, 203, 202);
+			init_pair(14, 212, 211);
+			init_pair(15, 216, 214);
+			init_pair(16, 201, 214);
+			init_pair(17, 204, 214);
+			init_pair(18, -1, -1);
+			init_pair(19, 201, 205);
+			set_colour(20, 218, 205, false, A_BOLD);
+			init_pair(21, 204, 207);
+			init_pair(28, 204, 211);
+
+			set_colour(22, COLOR_WHITE, 199, true, cursor_attr);
+			set_colour(23, 204, 199, false, 0);
+			set_colour(27, COLOR_WHITE, COLOR_BLACK, true, A_REVERSE);		/* input */
+
+			init_pair(24, 213, 217);
+			init_pair(25, 204, 217);
+
+			theme_attrs[4] = labels_attr;
+			theme_attrs[5] = cursor_attr;
+			theme_attrs[6] = cursor_attr;
+			theme_attrs[10] = cursor_attr;
+
+			set_colour(31, 200, 206, false, 0);						/* scrollbar background */
+			set_colour(32, 200, 205, false, 0);							/* scrollbar slider */
+
+			set_colour(34, 201, 202, false, 0);
+			set_colour(35, 213, 203, false, cursor_attr);
 			break;
 	}
 }
@@ -927,6 +1091,21 @@ initialize_theme(int theme, int window_identifier, bool is_tabular_fmt, bool no_
 {
 	memset(t, 0, sizeof(Theme));
 
+	/* selected content and cursor in selected area */
+	t->selection_attr = theme_attr(34);
+	t->selection_cursor_attr = theme_attr(35) ;
+
+	/* cross cursor - initial setting */
+	t->cross_cursor_attr = theme_attr(22);
+	t->cross_cursor_line_attr = theme_attr(23);
+
+	t->pattern_vertical_cursor_attr = theme_attr(24);
+	t->pattern_vertical_cursor_line_attr = theme_attr(25);
+
+	/* input bottom line and error bottom line colors */
+	t->error_attr = theme_attr(26);
+	t->input_attr = theme_attr(27);
+
 	switch (window_identifier)
 	{
 		case WINDOW_LUC:
@@ -1034,7 +1213,38 @@ initialize_theme(int theme, int window_identifier, bool is_tabular_fmt, bool no_
 			t->cursor_bookmark_attr = theme_attr(14);
 			t->pattern_data_attr = theme_attr(16);
 			t->cursor_bookmark_attr |= A_BOLD | A_REVERSE;
+			break;
 
+		case WINDOW_VSCROLLBAR:
+			t->scrollbar_arrow_attr = theme_attr(30);
+			t->scrollbar_attr = theme_attr(31);
+			t->scrollbar_slider_attr = theme_attr(32);
+			t->scrollbar_active_slider_attr = theme_attr(33);
+
+			switch (theme)
+			{
+				case 2:
+					t->scrollbar_slider_symbol = ACS_DIAMOND;
+					break;
+
+				case 12:
+					t->scrollbar_slider_symbol = ACS_BLOCK;
+					break;
+
+				case 0:
+					t->scrollbar_slider_symbol = ACS_BLOCK;
+					t->scrollbar_use_arrows = true;
+					break;
+
+				case 1: case 3: case 7: case 8:
+				case 13: case 14: case 16: case 15:
+					t->scrollbar_use_arrows = true;
+					break;
+
+				default:
+					t->scrollbar_slider_symbol = 0;
+					t->scrollbar_use_arrows = false;
+			}
 			break;
 	}
 
@@ -1044,14 +1254,4 @@ initialize_theme(int theme, int window_identifier, bool is_tabular_fmt, bool no_
 		t->pattern_line_attr = t->line_attr;
 	}
 
-	/* cross cursor - initial setting */
-	t->cross_cursor_attr = theme_attr(22);
-	t->cross_cursor_line_attr = theme_attr(23);
-
-	t->pattern_vertical_cursor_attr = theme_attr(24);
-	t->pattern_vertical_cursor_line_attr = theme_attr(25);
-
-	/* input bottom line and error bottom line colors */
-	t->error_attr = theme_attr(26);
-	t->input_attr = theme_attr(27);
 }
