@@ -13,17 +13,18 @@
 
 #include <ctype.h>
 #include <stdbool.h>
-
 #include "unicode.h"
 #include "string.h"
 
+inline static wchar_t utf8_to_unicode(const unsigned char *c);
+
 /*
  * Returns length of utf8 string in chars.
  */
-size_t
-utf8len(char *s)
+inline int
+utf8len(const char *s)
 {
-	size_t len = 0;
+	int len = 0;
 
 	for (; *s; ++s)
 		if ((*s & 0xC0) != 0x80)
@@ -49,7 +50,7 @@ utf8len_start_stop(const char *start, const char *stop)
  * Returns length of utf8 char in bytes
  */
 inline int
-utf8charlen(char ch)
+utf8charlen(const char ch)
 {
 	if ((ch & 0x80) == 0)
 		return 1;
@@ -85,7 +86,7 @@ struct mbinterval
 };
 
 /* auxiliary function for binary search in interval table */
-static int
+inline static int
 mbbisearch(wchar_t ucs, const struct mbinterval *table, int max)
 {
 	int			min = 0;
@@ -288,7 +289,7 @@ unicode_to_utf8(wchar_t c, unsigned char *utf8string, int *size)
  *
  * No error checks here, c must point to a long-enough string.
  */
-wchar_t
+inline static wchar_t
 utf8_to_unicode(const unsigned char *c)
 {
 	if ((*c & 0x80) == 0)
@@ -324,7 +325,7 @@ utf8_to_unicode(const unsigned char *c)
 inline int
 utf_dsplen(const char *s)
 {
-	if (*s == ' ')
+	if (*s >= 0x20 && *s < 0x7f)
 		return 1;
 
 	return ucs_wcwidth(utf8_to_unicode((const unsigned char *) s));
@@ -341,28 +342,31 @@ _utf_dsplen(const char *s)
  * The string is limited by max_bytes too.
  */
 int
-utf_string_dsplen(const char *s, size_t max_bytes)
+utf_string_dsplen(const char *s, int bytes)
 {
 	int result = 0;
-	const char *ptr = s;
 
-	while (*ptr != '\0' && max_bytes > 0)
+	while (bytes > 0)
 	{
-		if (*ptr == ' ')
-		{
-			ptr += 1;
-			result += 1;
-			max_bytes -= 1;
-		} 
-		else
-		{
-			int		clen;
+		const char c = *s;
 
-			clen  = utf8charlen(*ptr);
-			result += _utf_dsplen(ptr);
-			ptr += clen;
-			max_bytes -= clen;
+		if (c >= 0x20 && c < 0x7f)
+		{
+			s += 1;
+			result += 1;
+			bytes -= 1;
 		}
+		else if (c)
+		{
+			int clen  = utf8charlen(c);
+
+			result += _utf_dsplen(s);
+
+			s += clen;
+			bytes -= clen;
+		}
+		else
+			break;
 	}
 
 	return result;
@@ -633,6 +637,44 @@ utf8_nstrstr_with_sizes(const char *haystack,
 	return haystack;
 }
 
+bool
+utf8_nstarts_with_with_sizes(const char *str,
+							 int str_size,
+							 const char *pattern,
+							 int pattern_size)
+{
+	while (pattern_size > 0)
+	{
+		int		bytes_c;
+		int		bytes_p;
+
+		if (str_size <= 0)
+			return false;
+
+		bytes_c = utf8charlen(*str);
+		bytes_p = utf8charlen(*pattern);
+
+		if (bytes_c != bytes_p ||
+			memcmp(str, pattern, bytes_c) != 0)
+		{
+			int		f1, f2;
+
+			f1 = utf8_tofold(str);
+			f2 = utf8_tofold(pattern);
+
+			if (f1 != f2)
+				return false;
+		}
+
+		str += bytes_c;
+		str_size -= bytes_c;
+
+		pattern += bytes_p;
+		pattern_size -= bytes_p;
+	}
+
+	return true;
+}
 
 const char *
 utf8_nstrstr(const char *haystack, const char *needle)
